#!/usr/bin/env python
#
#  mod_dav_svn_tests.py:  testing mod_dav_svn
#
#  Subversion is a tool for revision control.
#  See http://subversion.apache.org for more information.
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################

# General modules
import os, logging, base64, functools

try:
  # Python <3.0
  import httplib
except ImportError:
  # Python >=3.0
  import http.client as httplib

logger = logging.getLogger()

# Our testing module
import svntest

# (abbreviation)
Skip = svntest.testcase.Skip_deco
SkipUnless = svntest.testcase.SkipUnless_deco
XFail = svntest.testcase.XFail_deco
Issues = svntest.testcase.Issues_deco
Issue = svntest.testcase.Issue_deco
Wimp = svntest.testcase.Wimp_deco

######################################################################
# Helper routines

def compare(lhs, rhs):
  """Implements cmp() for Python 2 and 3 alike"""
  if lhs == None:
    if rhs == None:
      return 0
    else:
      return -1
  else:
    if rhs == None:
      return 1
    else:
      return (lhs > rhs) - (lhs < rhs)

def compare_dict(lhs, rhs):
  """Implements dictionary comparison for Python 2 and 3 alike"""
  lhs_sorted = sorted(lhs, key=lambda x:sorted(x.keys()))
  rhs_sorted = sorted(rhs, key=lambda x:sorted(x.keys()))
  return (lhs_sorted > rhs_sorted) - (lhs_sorted < rhs_sorted)

def compare_xml_elem(a, b):
  """Recursively compare two xml.etree.ElementTree.Element objects.
  Return a 3-tuple made out of (cmp, elem_a, elem_b), where cmp is
  the integer result of the comparison (negative, zero or positive),
  and elem_a and elem_b point to mismatching elements.  Iff cmp is
  zero, elem_a and elem_b are None. """

  # Compare tags, attributes, inner text, tail attribute and the
  # number of child elements.
  res = compare(a.tag, b.tag)
  if res != 0:
    return res, a, b
  # Don't care about the order of the attributes.
  res = compare_dict(a.attrib, b.attrib)
  if res != 0:
    return res, a, b
  res = compare(a.text, b.text)
  if res != 0:
    return res, a, b
  res = compare(a.tail, b.tail)
  if res != 0:
    return res, a, b
  res = compare(len(a), len(b))
  if res != 0:
    return res, a, b

  # Prior to recursing, order child elements using the same comparator.
  # Right now we don't care about the order of the elements.  For instance,
  # <D:response>'s in PROPFIND *need* to be compared without a particular
  # order, since the server returns them in an unstable order of the hash
  # iteration.
  def sortcmp(x, y):
    return compare_xml_elem(x, y)[0]

  a_children = sorted(list(a), key=functools.cmp_to_key(sortcmp))
  b_children = sorted(list(b), key=functools.cmp_to_key(sortcmp))

  for a_child, b_child in zip(a_children, b_children):
    res = compare_xml_elem(a_child, b_child)
    if res[0] != 0:
      return res

  # Elements are equal.
  return 0, None, None

def verify_xml_response(expected_xml, actual_xml):
  """Parse and compare two XML responses, raise svntest.Failure
  in case EXPECTED_XML doesn't match ACTUAL_XML. """

  import xml.etree.ElementTree as ET

  expected_root = ET.fromstring(expected_xml)
  actual_root = ET.fromstring(actual_xml)
  res, expected_elem, actual_elem = compare_xml_elem(expected_root,
                                                     actual_root)
  if res != 0:
    # The actual response doesn't match our expectations; dump it for
    # debugging purposes, and highlight the mismatching xml element.
    logger.warn("Response:\n%s" % actual_xml)
    raise svntest.Failure("Unexpected response part\n"
                          "  Expected: '%s'\n    Actual: '%s'\n"
                          % (ET.tostring(expected_elem),
                             ET.tostring(actual_elem)))

######################################################################
# Tests

@SkipUnless(svntest.main.is_ra_type_dav)
def cache_control_header(sbox):
  "verify 'Cache-Control' headers on responses"

  sbox.build(create_wc=False, read_only=True)

  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
  }

  h = svntest.main.create_http_connection(sbox.repo_url)

  # GET /repos/iota
  # Response depends on the youngest revision in the repository, and
  # can't be cached; expect to see Cache-Control: max-age=0.
  h.request('GET', sbox.repo_url + '/iota', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=0',
                                           r.getheader('Cache-Control'))
  r.read()

  # GET /repos/A/
  # Response depends on the youngest revision in the repository, and
  # can't be cached; expect to see Cache-Control: max-age=0.
  h.request('GET', sbox.repo_url + '/A/', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=0',
                                           r.getheader('Cache-Control'))
  r.read()

  # GET /repos/A/?p=1
  # Response for a pegged directory is a subject for authz filtering, and
  # can't be cached; expect to see Cache-Control: max-age=0.
  h.request('GET', sbox.repo_url + '/A/?p=1', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=0',
                                           r.getheader('Cache-Control'))
  r.read()

  # GET /repos/iota?r=1
  # Response for a file URL with ?r=WORKINGREV is mutable, because the
  # line of history for this file can be replaced in the future (hence,
  # the same request will start producing another response).  Expect to
  # see Cache-Control: max-age=0.
  h.request('GET', sbox.repo_url + '/iota?r=1', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=0',
                                           r.getheader('Cache-Control'))
  r.read()

  # GET /repos/iota?p=1
  # Response for a pegged file is immutable; expect to see Cache-Control
  # with non-zero max-age.
  h.request('GET', sbox.repo_url + '/iota?p=1', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=604800',
                                           r.getheader('Cache-Control'))
  r.read()

  # GET /repos/iota?p=1&r=1
  # Response for a file URL with both ?p=PEG_REV and ?r=WORKINGREV is
  # immutable; expect to see Cache-Control with non-zero max-age.
  h.request('GET', sbox.repo_url + '/iota?p=1&r=1', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=604800',
                                           r.getheader('Cache-Control'))
  r.read()


  # GET /repos/!svn/rvr/1/iota
  # Response is immutable; expect to see Cache-Control with non-zero max-age.
  h.request('GET', sbox.repo_url + '/!svn/rvr/1/iota', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Cache-Control',
                                           'max-age=604800',
                                           r.getheader('Cache-Control'))
  r.read()


@SkipUnless(svntest.main.is_ra_type_dav)
def simple_propfind(sbox):
  "verify simple PROPFIND responses"

  sbox.build(create_wc=False, read_only=True)
  repo_uripath = '/' + svntest.wc.svn_uri_quote(
    svntest.main.pristine_greek_repos_dir.replace(os.path.sep, '/'))
  h = svntest.main.create_http_connection(sbox.repo_url)

  # PROPFIND /repos/!svn/rvr/1, Depth = 0
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '0',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
    '<prop><resourcetype xmlns="DAV:"/></prop>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:lp1="DAV:" '
        'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype><D:collection/></lp1:resourcetype>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)

  # PROPFIND /repos/!svn/rvr/1, Depth = 1
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '1',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
      '<prop><resourcetype xmlns="DAV:"/></prop>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:lp1="DAV:" '
        'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype><D:collection/></lp1:resourcetype>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
      '<D:response xmlns:lp1="DAV:" '
        'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/A/</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype><D:collection/></lp1:resourcetype>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
      '<D:response xmlns:lp1="DAV:" '
        'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/iota</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype/>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)

  # PROPFIND /repos/!svn/rvr/1/A/B/F, Depth = 1
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '1',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
      '<prop><resourcetype xmlns="DAV:"/></prop>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1/A/B/F', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:lp1="DAV:" '
        'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/A/B/F/</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype><D:collection/></lp1:resourcetype>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)

  # PROPFIND /repos/!svn/rvr/1/iota, Depth = 0
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '0',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
      '<prop><resourcetype xmlns="DAV:"/></prop>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1/iota', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:lp1="DAV:" '
        'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/iota</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype/>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)


@SkipUnless(svntest.main.is_ra_type_dav)
def propfind_multiple_props(sbox):
  "verify multi-prop PROPFIND response"

  sbox.build(create_wc=False, read_only=True)
  repo_uripath = '/' + svntest.wc.svn_uri_quote(
    svntest.main.pristine_greek_repos_dir.replace(os.path.sep, '/'))
  h = svntest.main.create_http_connection(sbox.repo_url)

  # PROPFIND /repos/!svn/rvr/1/iota, Depth = 0
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '0',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8" ?>\n'
    '<D:propfind xmlns:D="DAV:">\n'
      '<D:prop xmlns:S="http://subversion.tigris.org/xmlns/dav/">\n'
         '<D:resourcetype/>\n'
         '<S:md5-checksum/>\n'
      '</D:prop>\n'
    '</D:propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1/iota', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" '
                   'xmlns:ns1="http://subversion.tigris.org/xmlns/dav/" '
                   'xmlns:ns0="DAV:">\n'
      '<D:response xmlns:lp1="DAV:" '
                  'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
      '<D:href>' + repo_uripath + '/!svn/rvr/1/iota</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype/>\n'
            '<lp2:md5-checksum>'
              '2d18c5e57e84c5b8a5e9a6e13fa394dc'
            '</lp2:md5-checksum>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)


@SkipUnless(svntest.main.is_ra_type_dav)
def propfind_404(sbox):
  "verify PROPFIND for non-existing property"

  sbox.build(create_wc=False, read_only=True)
  repo_uripath = '/' + svntest.wc.svn_uri_quote(
    svntest.main.pristine_greek_repos_dir.replace(os.path.sep, '/'))
  h = svntest.main.create_http_connection(sbox.repo_url)

  # PROPFIND /repos/!svn/rvr/1, Depth = 0
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '0',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
      '<prop><nonexistingprop xmlns="DAV:"/></prop>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:g0="DAV:">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<g0:nonexistingprop/>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 404 Not Found</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)


@SkipUnless(svntest.main.is_ra_type_dav)
def propfind_allprop(sbox):
  "verify allprop PROPFIND response"

  sbox.build()
  repo_uripath = '/' + svntest.wc.svn_uri_quote(
    sbox.repo_dir.replace(os.path.sep, '/'))
  svntest.actions.enable_revprop_changes(sbox.repo_dir)
  # Ensure stable date and uuid
  svntest.main.run_svnadmin('setuuid', sbox.repo_dir,
                            'd7130b12-92f6-45c9-9217-b9f0472c3fab')
  svntest.actions.run_and_verify_svn(None, [],
                                     'propset', '--revprop', '-r', '1',
                                     'svn:date', '2015-01-01T00:00:00.0Z',
                                     sbox.wc_dir)

  h = svntest.main.create_http_connection(sbox.repo_url)

  # PROPFIND /repos/!svn/rvr/1, Depth = 0
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '0',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
      '<allprop/>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/1', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:S="http://subversion.tigris.org/xmlns/svn/" '
                  'xmlns:C="http://subversion.tigris.org/xmlns/custom/" '
                  'xmlns:V="http://subversion.tigris.org/xmlns/dav/" '
                  'xmlns:lp1="DAV:" '
                  'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/1/</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
            '<lp1:resourcetype><D:collection/></lp1:resourcetype>\n'
            '<lp1:getcontenttype>' +
              'text/html; charset=UTF-8' +
            '</lp1:getcontenttype>\n'
            '<lp1:getetag>W/"1//"</lp1:getetag>\n'
            '<lp1:creationdate>2015-01-01T00:00:00.0Z</lp1:creationdate>\n'
            '<lp1:getlastmodified>' +
              'Thu, 01 Jan 2015 00:00:00 GMT' +
            '</lp1:getlastmodified>\n'
            '<lp1:checked-in>'
              '<D:href>' + repo_uripath + '/!svn/ver/1/</D:href>'
            '</lp1:checked-in>\n'
            '<lp1:version-controlled-configuration>'
              '<D:href>' + repo_uripath + '/!svn/vcc/default</D:href>'
            '</lp1:version-controlled-configuration>\n'
            '<lp1:version-name>1</lp1:version-name>\n'
            '<lp1:creator-displayname>jrandom</lp1:creator-displayname>\n'
            '<lp2:baseline-relative-path/>\n'
            '<lp2:repository-uuid>' +
              'd7130b12-92f6-45c9-9217-b9f0472c3fab' +
            '</lp2:repository-uuid>\n'
            '<lp2:deadprop-count>0</lp2:deadprop-count>\n'
            '<D:lockdiscovery/>\n'
          '</D:prop>\n'
        '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)


@SkipUnless(svntest.main.is_ra_type_dav)
def propfind_propname(sbox):
  "verify propname PROPFIND response"

  sbox.build()
  sbox.simple_propset('a', 'b', 'iota')
  sbox.simple_commit()
  repo_uripath = '/' + svntest.wc.svn_uri_quote(
    sbox.repo_dir.replace(os.path.sep, '/'))

  h = svntest.main.create_http_connection(sbox.repo_url)

  # PROPFIND /repos/!svn/rvr/2/iota, Depth = 0
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Depth': '0',
  }
  req_body = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<propfind xmlns="DAV:">\n'
      '<propname/>\n'
    '</propfind>\n'
    )
  h.request('PROPFIND', sbox.repo_url + '/!svn/rvr/2/iota', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.MULTI_STATUS:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))

  expected_response = (
    '<?xml version="1.0" encoding="utf-8"?>\n'
    '<D:multistatus xmlns:D="DAV:" xmlns:ns0="DAV:">\n'
      '<D:response xmlns:S="http://subversion.tigris.org/xmlns/svn/" '
                  'xmlns:C="http://subversion.tigris.org/xmlns/custom/" '
                  'xmlns:V="http://subversion.tigris.org/xmlns/dav/" '
                  'xmlns:lp1="DAV:" '
                  'xmlns:lp2="http://subversion.tigris.org/xmlns/dav/">\n'
        '<D:href>' + repo_uripath + '/!svn/rvr/2/iota</D:href>\n'
        '<D:propstat>\n'
          '<D:prop>\n'
          '<C:a/>\n'
          '<lp1:resourcetype/>\n'
          '<lp1:getcontentlength/>\n'
          '<lp1:getcontenttype/>\n'
          '<lp1:getetag/>\n'
          '<lp1:creationdate/>\n'
          '<lp1:getlastmodified/>\n'
          '<lp1:checked-in/>\n'
          '<lp1:version-controlled-configuration/>\n'
          '<lp1:version-name/>\n'
          '<lp1:creator-displayname/>\n'
          '<lp2:baseline-relative-path/>\n'
          '<lp2:md5-checksum/>\n'
          '<lp2:repository-uuid/>\n'
          '<lp2:deadprop-count/>\n'
          '<lp2:sha1-checksum/>\n'
          '<D:supportedlock/>\n'
          '<D:lockdiscovery/>\n'
          '</D:prop>\n'
          '<D:status>HTTP/1.1 200 OK</D:status>\n'
        '</D:propstat>\n'
      '</D:response>\n'
    '</D:multistatus>\n'
    )
  actual_response = r.read()
  verify_xml_response(expected_response, actual_response)

@SkipUnless(svntest.main.is_ra_type_dav)
def last_modified_header(sbox):
  "verify 'Last-Modified' header on 'external' GETs"

  sbox.build(create_wc=False, read_only=True)

  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
  }

  h = svntest.main.create_http_connection(sbox.repo_url)

  # GET /repos/iota
  # Expect to see a Last-Modified header.
  h.request('GET', sbox.repo_url + '/iota', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Last-Modified',
                                           svntest.verify.RegexOutput('.+'),
                                           r.getheader('Last-Modified'))
  r.read()

  # HEAD /repos/iota
  # Expect to see a Last-Modified header.
  h.request('HEAD', sbox.repo_url + '/iota', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  svntest.verify.compare_and_display_lines(None, 'Last-Modified',
                                           svntest.verify.RegexOutput('.+'),
                                           r.getheader('Last-Modified'))
  r.read()

  # GET /repos/!svn/rvr/1/iota
  # There should not be a Last-Modified header (it's costly and not useful,
  # see r1724790)
  h.request('GET', sbox.repo_url + '/!svn/rvr/1/iota', None, headers)
  r = h.getresponse()
  if r.status != httplib.OK:
    raise svntest.Failure('Request failed: %d %s' % (r.status, r.reason))
  last_modified = r.getheader('Last-Modified')
  if last_modified:
    raise svntest.Failure('Unexpected Last-Modified header: %s' % last_modified)
  r.read()

@SkipUnless(svntest.main.is_ra_type_dav)
def create_name_with_control_chars(sbox):
  "test creating items with control chars in names"

  sbox.build(create_wc=False)

  h = svntest.main.create_http_connection(sbox.repo_url)

  # POST /repos/!svn/me
  # Create a new transaction.
  req_body = (
    '(create-txn-with-props '
    '(svn:txn-client-compat-version 6 1.14.4 '
    'svn:txn-user-agent 45 SVN/1.14.4 (x86-microsoft-windows) serf/1.3.9 '
    'svn:log 0 ))'
    )
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Content-Type': 'application/vnd.svn-skel',
  }
  h.request('POST', sbox.repo_url + '/!svn/me', req_body, headers)
  r = h.getresponse()
  if r.status != httplib.CREATED:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))
  txn_name = r.getheader('SVN-Txn-Name')
  r.read()

  # MKCOL /repos/!svn/txn/TXN_NAME/tab%09name
  # Must fail with a 400 Bad Request.
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
  }
  h.request('MKCOL', sbox.repo_url + '/!svn/txr/' + txn_name + '/tab%09name', None, headers)
  r = h.getresponse()
  if r.status != httplib.BAD_REQUEST:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))
  r.read()

  # PUT /repos/!svn/txn/TXN_NAME/tab%09name
  # Must fail with a 400 Bad Request.
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
  }
  h.request('PUT', sbox.repo_url + '/!svn/txr/' + txn_name + '/tab%09name', None, headers)
  r = h.getresponse()
  if r.status != httplib.BAD_REQUEST:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))
  r.read()

  # COPY /repos/!svn/rvr/1/iota -> /repos/!svn/txn/TXN_NAME/tab%09name
  # Must fail with a 400 Bad Request.
  headers = {
    'Authorization': 'Basic ' + base64.b64encode(b'jconstant:rayjandom').decode(),
    'Destination': sbox.repo_url + '/!svn/txr/' + txn_name + '/tab%09name'
  }
  h.request('COPY', sbox.repo_url + '/!svn/rvr/1/iota', None, headers)
  r = h.getresponse()
  if r.status != httplib.BAD_REQUEST:
    raise svntest.Failure('Unexpected status: %d %s' % (r.status, r.reason))
  r.read()


########################################################################
# Run the tests


# list all tests here, starting with None:
test_list = [ None,
              cache_control_header,
              simple_propfind,
              propfind_multiple_props,
              propfind_404,
              propfind_allprop,
              propfind_propname,
              last_modified_header,
              create_name_with_control_chars,
             ]
serial_only = True

if __name__ == '__main__':
  svntest.main.run_tests(test_list)
  # NOTREACHED


### End of file.



#!/usr/bin/env python

# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
# ====================================================================

args_message = '[-f png|svg|gif|dia... [-f ...]] GRAPH_CONFIG_FILE...'
help_message = """Produce pretty graphs representing branches and merging.
For each config file specified, construct a graph and write it as a PNG file
(or other graphical file formats)."""

import sys
import getopt
from mergegraph import MergeDot


# If run as a program, process each input filename as a graph config file.
if __name__ == '__main__':
  optlist, args = getopt.getopt(sys.argv[1:], 'f:', ['format'])

  prog_name = sys.argv[0]
  if not args:
    usage = '%s: usage: "%s %s"\n' % (prog_name, prog_name, args_message)
    sys.stderr.write(usage)
    sys.exit(1)

  formats = []

  for opt, opt_arg in optlist:
    if opt == '-f':
      formats.append(opt_arg)

  if not formats:
    formats.append('png')

  for config_filename in args:
    sys.stdout.write("%s: reading '%s', " % (prog_name, config_filename))
    graph = MergeDot(config_filename, rankdir='LR', dpi='72')
    for format in formats:
      filename = '%s.%s' % (graph.basename, format)
      sys.stdout.write("writing '%s' " % filename)
      graph.save(format=format, filename=filename)
    print



#!/usr/bin/env python
#
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#
"""ezt.py -- easy templating

ezt templates are simply text files in whatever format you so desire
(such as XML, HTML, etc.) which contain directives sprinkled
throughout.  With these directives it is possible to generate the
dynamic content from the ezt templates.

These directives are enclosed in square brackets.  If you are a
C-programmer, you might be familar with the #ifdef directives of the C
preprocessor 'cpp'.  ezt provides a similar concept.  Additionally EZT
has a 'for' directive, which allows it to iterate (repeat) certain
subsections of the template according to sequence of data items
provided by the application.

The final rendering is performed by the method generate() of the Template
class.  Building template instances can either be done using external
EZT files (convention: use the suffix .ezt for such files):

    >>> template = Template("../templates/log.ezt")

or by calling the parse() method of a template instance directly with
a EZT template string:

    >>> template = Template()
    >>> template.parse('''<html><head>
    ... <title>[title_string]</title></head>
    ... <body><h1>[title_string]</h1>
    ...    [for a_sequence] <p>[a_sequence]</p>
    ...    [end] <hr>
    ...    The [person] is [if-any state]in[else]out[end].
    ... </body>
    ... </html>
    ... ''')

The application should build a dictionary 'data' and pass it together
with the output fileobject to the templates generate method:

    >>> data = {'title_string' : "A Dummy Page",
    ...         'a_sequence' : ['list item 1', 'list item 2', 'another element'],
    ...         'person': "doctor",
    ...         'state' : None }
    >>> import sys
    >>> template.generate(sys.stdout, data)
    <html><head>
    <title>A Dummy Page</title></head>
    <body><h1>A Dummy Page</h1>
     <p>list item 1</p>
     <p>list item 2</p>
     <p>another element</p>
     <hr>
    The doctor is out.
    </body>
    </html>

Template syntax error reporting should be improved.  Currently it is
very sparse (template line numbers would be nice):

    >>> Template().parse("[if-any where] foo [else] bar [end unexpected args]")
    Traceback (innermost last):
      File "<stdin>", line 1, in ?
      File "ezt.py", line 220, in parse
        self.program = self._parse(text)
      File "ezt.py", line 275, in _parse
        raise ArgCountSyntaxError(str(args[1:]))
    ArgCountSyntaxError: ['unexpected', 'args']
    >>> Template().parse("[if unmatched_end]foo[end]")
    Traceback (innermost last):
      File "<stdin>", line 1, in ?
      File "ezt.py", line 206, in parse
        self.program = self._parse(text)
      File "ezt.py", line 266, in _parse
        raise UnmatchedEndError()
    UnmatchedEndError


Directives
==========

 Several directives allow the use of dotted qualified names refering to objects
 or attributes of objects contained in the data dictionary given to the
 .generate() method.

 Qualified names
 ---------------

   Qualified names have two basic forms: a variable reference, or a string
   constant. References are a name from the data dictionary with optional
   dotted attributes (where each intermediary is an object with attributes,
   of course).

   Examples:

     [varname]

     [ob.attr]

     ["string"]

 Simple directives
 -----------------

   [QUAL_NAME]

   This directive is simply replaced by the value of the qualified name.
   Numbers are converted to a string, and None becomes an empty string.

   [QUAL_NAME QUAL_NAME ...]

   The first value defines a substitution format, specifying constant
   text and indices of the additional arguments. The arguments are then
   substituted and the resulting is inserted into the output stream.

   Example:
     ["abc %0 def %1 ghi %0" foo bar.baz]

   Note that the first value can be any type of qualified name -- a string
   constant or a variable reference. Use %% to substitute a percent sign.
   Argument indices are 0-based.

   [include "filename"]  or [include QUAL_NAME]

   This directive is replaced by content of the named include file. Note
   that a string constant is more efficient -- the target file is compiled
   inline. In the variable form, the target file is compiled and executed
   at runtime.

   [insertfile "filename"] or [insertfile QUAL_NAME]

   This directive is replace by content from the named file, but as a
   literal string: directives in the target file are not expanded.  As
   in the case of the "include" directive, using a string constant for
   the filename is more efficient than the variable form.

 Block directives
 ----------------

   [for QUAL_NAME] ... [end]

   The text within the [for ...] directive and the corresponding [end]
   is repeated for each element in the sequence referred to by the
   qualified name in the for directive.  Within the for block this
   identifiers now refers to the actual item indexed by this loop
   iteration.

   [if-any QUAL_NAME [QUAL_NAME2 ...]] ... [else] ... [end]

   Test if any QUAL_NAME value is not None or an empty string or list.
   The [else] clause is optional.  CAUTION: Numeric values are
   converted to string, so if QUAL_NAME refers to a numeric value 0,
   the then-clause is substituted!

   [if-index INDEX_FROM_FOR odd] ... [else] ... [end]
   [if-index INDEX_FROM_FOR even] ... [else] ... [end]
   [if-index INDEX_FROM_FOR first] ... [else] ... [end]
   [if-index INDEX_FROM_FOR last] ... [else] ... [end]
   [if-index INDEX_FROM_FOR NUMBER] ... [else] ... [end]

   These five directives work similar to [if-any], but are only useful
   within a [for ...]-block (see above).  The odd/even directives are
   for example useful to choose different background colors for
   adjacent rows in a table.  Similar the first/last directives might
   be used to remove certain parts (for example "Diff to previous"
   doesn't make sense, if there is no previous).

   [is QUAL_NAME STRING] ... [else] ... [end]
   [is QUAL_NAME QUAL_NAME] ... [else] ... [end]

   The [is ...] directive is similar to the other conditional
   directives above.  But it allows to compare two value references or
   a value reference with some constant string.

   [define VARIABLE] ... [end]

   The [define ...] directive allows you to create and modify template
   variables from within the template itself.  Essentially, any data
   between inside the [define ...] and its matching [end] will be
   expanded using the other template parsing and output generation
   rules, and then stored as a string value assigned to the variable
   VARIABLE.  The new (or changed) variable is then available for use
   with other mechanisms such as [is ...] or [if-any ...], as long as
   they appear later in the template.

   [format "html|xml|js|url|raw"] ... [end]

   The [format ...] directive creates a block in which any substitutions
   are processed as though the template has been instantiated with the
   the corresponding 'base_format' argument. Comma-separated format
   specifiers perform nested encodings. In this case the encodings are
   applied left-to-right.  For example the directive: [format "html,js"]
   will HTML and then Javascript encode any inserted template variables.
"""
#
# Copyright (C) 2001-2009 Greg Stein. All Rights Reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#
# This software is maintained by Greg and is available at:
#    http://code.google.com/p/ezt/
#

import os, re, sys

if sys.version_info[0] >= 3:
  # Python >=3.0
  long = int
  unicode = str
  from io import StringIO
  from urllib.parse import quote_plus as urllib_parse_quote_plus
else:
  # Python <3.0
  from urllib import quote_plus as urllib_parse_quote_plus
  try:
    from cStringIO import StringIO
  except ImportError:
    from StringIO import StringIO

#
# Formatting types
#
FORMAT_RAW = 'raw'
FORMAT_HTML = 'html'
FORMAT_XML = 'xml'
FORMAT_JS = 'js'
FORMAT_URL = 'url'

#
# This regular expression matches three alternatives:
#   expr: NEWLINE | DIRECTIVE | BRACKET | COMMENT
#   DIRECTIVE: '[' ITEM (whitespace ITEM)* ']
#   ITEM: STRING | NAME
#   STRING: '"' (not-slash-or-dquote | '\' anychar)* '"'
#   NAME: (alphanum | '_' | '-' | '.')+
#   BRACKET: '[[]'
#   COMMENT: '[#' not-rbracket* ']'
#
# When used with the split() method, the return value will be composed of
# non-matching text and the three paren groups (NEWLINE, DIRECTIVE and
# BRACKET). Since the COMMENT matches are not placed into a group, they are
# considered a "splitting" value and simply dropped.
#
_item = r'(?:"(?:[^\\"]|\\.)*"|[-\w.]+)'
_re_parse = re.compile(r'(\r?\n)|\[(%s(?: +%s)*)\]|(\[\[\])|\[#[^\]]*\]' %
                       (_item, _item))

_re_args = re.compile(r'"(?:[^\\"]|\\.)*"|[-\w.]+')

# block commands and their argument counts
_block_cmd_specs = { 'if-index':2, 'for':1, 'is':2, 'define':1, 'format':1 }
_block_cmds = _block_cmd_specs.keys()

# two regular expressions for compressing whitespace. the first is used to
# compress any whitespace including a newline into a single newline. the
# second regex is used to compress runs of whitespace into a single space.
_re_newline = re.compile('[ \t\r\f\v]*\n\\s*')
_re_whitespace = re.compile(r'\s\s+')

# this regex is used to substitute arguments into a value. we split the value,
# replace the relevant pieces, and then put it all back together. splitting
# will produce a list of: TEXT ( splitter TEXT )*. splitter will be '%' or
# an integer.
_re_subst = re.compile('%(%|[0-9]+)')

class Template:

  def __init__(self, fname=None, compress_whitespace=1,
               base_format=FORMAT_RAW):
    self.compress_whitespace = compress_whitespace
    if fname:
      self.parse_file(fname, base_format)

  def parse_file(self, fname, base_format=FORMAT_RAW):
    "fname -> a string object with pathname of file containg an EZT template."

    self.parse(_FileReader(fname), base_format)

  def parse(self, text_or_reader, base_format=FORMAT_RAW):
    """Parse the template specified by text_or_reader.

    The argument should be a string containing the template, or it should
    specify a subclass of ezt.Reader which can read templates. The base
    format for printing values is given by base_format.
    """
    if not isinstance(text_or_reader, Reader):
      # assume the argument is a plain text string
      text_or_reader = _TextReader(text_or_reader)

    self.program = self._parse(text_or_reader,
                               base_printer=_parse_format(base_format))

  def generate(self, fp, data):
    if hasattr(data, '__getitem__') or hasattr(getattr(data, 'keys', None), '__call__'):
      # a dictionary-like object was passed. convert it to an
      # attribute-based object.
      class _data_ob:
        def __init__(self, d):
          vars(self).update(d)
      data = _data_ob(data)

    ctx = _context()
    ctx.data = data
    ctx.for_index = { }
    ctx.defines = { }
    self._execute(self.program, fp, ctx)

  def _parse(self, reader, for_names=None, file_args=(), base_printer=None):
    """text -> string object containing the template.

    This is a private helper function doing the real work for method parse.
    It returns the parsed template as a 'program'.  This program is a sequence
    made out of strings or (function, argument) 2-tuples.

    Note: comment directives [# ...] are automatically dropped by _re_parse.
    """

    filename = reader.filename()
    # parse the template program into: (TEXT NEWLINE DIRECTIVE BRACKET)* TEXT
    parts = _re_parse.split(reader.text)

    program = [ ]
    stack = [ ]
    if not for_names:
      for_names = [ ]

    if base_printer is None:
      base_printer = ()
    printers = [ base_printer ]

    one_newline_copied = False
    line_number = 1
    for i in range(len(parts)):
      piece = parts[i]
      which = i % 4  # discriminate between: TEXT NEWLINE DIRECTIVE BRACKET
      if which == 0:
        # TEXT. append if non-empty.
        if piece:
          if self.compress_whitespace:
            piece = _re_whitespace.sub(' ', piece)
          program.append(piece)
          one_newline_copied = False
      elif which == 1:
        # NEWLINE. append unless compress_whitespace requested
        if piece:
          line_number += 1
          if self.compress_whitespace:
            if not one_newline_copied:
              program.append('\n')
              one_newline_copied = True
          else:
            program.append(piece)
      elif which == 3:
        # BRACKET directive. append '[' if present.
        if piece:
          program.append('[')
          one_newline_copied = False
      elif piece:
        # DIRECTIVE is present.
        one_newline_copied = False
        args = _re_args.findall(piece)
        cmd = args[0]
        if cmd == 'else':
          if len(args) > 1:
            raise ArgCountSyntaxError(str(args[1:]), filename, line_number)
          ### check: don't allow for 'for' cmd
          idx = stack[-1][1]
          true_section = program[idx:]
          del program[idx:]
          stack[-1][3] = true_section
        elif cmd == 'end':
          if len(args) > 1:
            raise ArgCountSyntaxError(str(args[1:]), filename, line_number)
          # note: true-section may be None
          try:
            cmd, idx, args, true_section, start_line_number = stack.pop()
          except IndexError:
            raise UnmatchedEndError(None, filename, line_number)
          else_section = program[idx:]
          if cmd == 'format':
            printers.pop()
          else:
            func = getattr(self, '_cmd_' + re.sub('-', '_', cmd))
            program[idx:] = [ (func, (args, true_section, else_section),
                               filename, line_number) ]
            if cmd == 'for':
              for_names.pop()
        elif cmd in _block_cmds:
          if len(args) > _block_cmd_specs[cmd] + 1:
            raise ArgCountSyntaxError(str(args[1:]), filename, line_number)
          ### this assumes arg1 is always a ref unless cmd is 'define'
          if cmd != 'define':
            args[1] = _prepare_ref(args[1], for_names, file_args)

          # handle arg2 for the 'is' command
          if cmd == 'is':
            args[2] = _prepare_ref(args[2], for_names, file_args)
          elif cmd == 'for':
            for_names.append(args[1][0])  # append the refname
          elif cmd == 'format':
            if args[1][0]:
              raise BadFormatConstantError(str(args[1:]), filename, line_number)
            printers.append(_parse_format(args[1][1]))

          # remember the cmd, current pos, args, and a section placeholder
          stack.append([cmd, len(program), args[1:], None, line_number])
        elif cmd == 'include' or cmd == 'insertfile':
          is_insertfile = (cmd == 'insertfile')
          # extra arguments are meaningless when using insertfile
          if is_insertfile and len(args) != 2:
            raise ArgCountSyntaxError(str(args), filename, line_number)
          if args[1][0] == '"':
            include_filename = args[1][1:-1]
            if is_insertfile:
              program.append(reader.read_other(include_filename).text)
            else:
              f_args = [ ]
              for arg in args[2:]:
                f_args.append(_prepare_ref(arg, for_names, file_args))
              program.extend(self._parse(reader.read_other(include_filename),
                                         for_names, f_args, printers[-1]))
          else:
            if len(args) != 2:
              raise ArgCountSyntaxError(str(args), filename, line_number)
            if is_insertfile:
              cmd = self._cmd_insertfile
            else:
              cmd = self._cmd_include
            program.append((cmd,
                            (_prepare_ref(args[1], for_names, file_args),
                             reader, printers[-1]), filename, line_number))
        elif cmd == 'if-any':
          f_args = [ ]
          for arg in args[1:]:
            f_args.append(_prepare_ref(arg, for_names, file_args))
          stack.append(['if-any', len(program), f_args, None, line_number])
        else:
          # implied PRINT command
          if len(args) > 1:
            f_args = [ ]
            for arg in args:
              f_args.append(_prepare_ref(arg, for_names, file_args))
            program.append((self._cmd_subst,
                            (printers[-1], f_args[0], f_args[1:]),
                            filename, line_number))
          else:
            valref = _prepare_ref(args[0], for_names, file_args)
            program.append((self._cmd_print, (printers[-1], valref),
                            filename, line_number))

    if stack:
      raise UnclosedBlocksError('Block opened at line %s' % stack[-1][4],
                                filename=filename)
    return program

  def _execute(self, program, fp, ctx):
    """This private helper function takes a 'program' sequence as created
    by the method '_parse' and executes it step by step.  strings are written
    to the file object 'fp' and functions are called.
    """
    for step in program:
      if isinstance(step, str):
        fp.write(step)
      else:
        method, method_args, filename, line_number = step
        method(method_args, fp, ctx, filename, line_number)

  def _cmd_print(self, transforms_valref, fp, ctx, filename, line_number):
    (transforms, valref) = transforms_valref
    value = _get_value(valref, ctx, filename, line_number)
    # if the value has a 'read' attribute, then it is a stream: copy it
    if hasattr(value, 'read'):
      while 1:
        chunk = value.read(16384)
        if not chunk:
          break
        for t in transforms:
          chunk = t(chunk)
        fp.write(chunk)
    else:
      for t in transforms:
        value = t(value)
      fp.write(value)

  def _cmd_subst(self, transforms_valref_args, fp, ctx, filename,
                 line_number):
    (transforms, valref, args) = transforms_valref_args
    fmt = _get_value(valref, ctx, filename, line_number)
    parts = _re_subst.split(fmt)
    for i in range(len(parts)):
      piece = parts[i]
      if i%2 == 1 and piece != '%':
        idx = int(piece)
        if idx < len(args):
          piece = _get_value(args[idx], ctx, filename, line_number)
        else:
          piece = '<undef>'
      for t in transforms:
        piece = t(piece)
      fp.write(piece)

  def _cmd_include(self, valref_reader_printer, fp, ctx, filename,
                   line_number):
    (valref, reader, printer) = valref_reader_printer
    fname = _get_value(valref, ctx, filename, line_number)
    ### note: we don't have the set of for_names to pass into this parse.
    ### I don't think there is anything to do but document it
    self._execute(self._parse(reader.read_other(fname), base_printer=printer),
                  fp, ctx)

  def _cmd_insertfile(self, valref_reader_printer, fp, ctx, filename,
                      line_number):
    (valref, reader, printer) = valref_reader_printer
    fname = _get_value(valref, ctx, filename, line_number)
    fp.write(reader.read_other(fname).text)

  def _cmd_if_any(self, args, fp, ctx, filename, line_number):
    "If any value is a non-empty string or non-empty list, then T else F."
    (valrefs, t_section, f_section) = args
    value = 0
    for valref in valrefs:
      if _get_value(valref, ctx, filename, line_number):
        value = 1
        break
    self._do_if(value, t_section, f_section, fp, ctx)

  def _cmd_if_index(self, args, fp, ctx, filename, line_number):
    ((valref, value), t_section, f_section) = args
    list, idx = ctx.for_index[valref[0]]
    if value == 'even':
      value = idx % 2 == 0
    elif value == 'odd':
      value = idx % 2 == 1
    elif value == 'first':
      value = idx == 0
    elif value == 'last':
      value = idx == len(list)-1
    else:
      value = idx == int(value)
    self._do_if(value, t_section, f_section, fp, ctx)

  def _cmd_is(self, args, fp, ctx, filename, line_number):
    ((left_ref, right_ref), t_section, f_section) = args
    right_value = _get_value(right_ref, ctx, filename, line_number)
    left_value = _get_value(left_ref, ctx, filename, line_number)
    value = left_value.lower() == right_value.lower()
    self._do_if(value, t_section, f_section, fp, ctx)

  def _do_if(self, value, t_section, f_section, fp, ctx):
    if t_section is None:
      t_section = f_section
      f_section = None
    if value:
      section = t_section
    else:
      section = f_section
    if section is not None:
      self._execute(section, fp, ctx)

  def _cmd_for(self, args, fp, ctx, filename, line_number):
    ((valref,), unused, section) = args
    list = _get_value(valref, ctx, filename, line_number)
    refname = valref[0]
    if isinstance(list, str):
      raise NeedSequenceError(refname, filename, line_number)
    ctx.for_index[refname] = idx = [ list, 0 ]
    for item in list:
      self._execute(section, fp, ctx)
      idx[1] = idx[1] + 1
    del ctx.for_index[refname]

  def _cmd_define(self, args, fp, ctx, filename, line_number):
    ((name,), unused, section) = args
    valfp = StringIO()
    if section is not None:
      self._execute(section, valfp, ctx)
    ctx.defines[name] = valfp.getvalue()

def boolean(value):
  "Return a value suitable for [if-any bool_var] usage in a template."
  if value:
    return 'yes'
  return None


def _prepare_ref(refname, for_names, file_args):
  """refname -> a string containing a dotted identifier. example:"foo.bar.bang"
  for_names -> a list of active for sequences.

  Returns a `value reference', a 3-tuple made out of (refname, start, rest),
  for fast access later.
  """
  # is the reference a string constant?
  if refname[0] == '"':
    return None, refname[1:-1], None

  parts = refname.split('.')
  start = parts[0]
  rest = parts[1:]

  # if this is an include-argument, then just return the prepared ref
  if start[:3] == 'arg':
    try:
      idx = int(start[3:])
    except ValueError:
      pass
    else:
      if idx < len(file_args):
        orig_refname, start, more_rest = file_args[idx]
        if more_rest is None:
          # the include-argument was a string constant
          return None, start, None

        # prepend the argument's "rest" for our further processing
        rest[:0] = more_rest

        # rewrite the refname to ensure that any potential 'for' processing
        # has the correct name
        ### this can make it hard for debugging include files since we lose
        ### the 'argNNN' names
        if not rest:
          return start, start, [ ]
        refname = start + '.' + '.'.join(rest)

  if for_names:
    # From last to first part, check if this reference is part of a for loop
    for i in range(len(parts), 0, -1):
      name = '.'.join(parts[:i])
      if name in for_names:
        return refname, name, parts[i:]

  return refname, start, rest

def _get_value(refname_start_rest, ctx, filename, line_number):
  """refname_start_rest -> a prepared `value reference' (see above).
  ctx -> an execution context instance.

  Does a name space lookup within the template name space.  Active
  for blocks take precedence over data dictionary members with the
  same name.
  """
  (refname, start, rest) = refname_start_rest
  if rest is None:
    # it was a string constant
    return start

  # get the starting object
  if start in ctx.for_index:
    list, idx = ctx.for_index[start]
    ob = list[idx]
  elif start in ctx.defines:
    ob = ctx.defines[start]
  elif hasattr(ctx.data, start):
    ob = getattr(ctx.data, start)
  else:
    raise UnknownReference(refname, filename, line_number)

  # walk the rest of the dotted reference
  for attr in rest:
    try:
      ob = getattr(ob, attr)
    except AttributeError:
      raise UnknownReference(refname, filename, line_number)

  # make sure we return a string instead of some various Python types
  if isinstance(ob, (int, long, float)):
    return str(ob)
  if ob is None:
    return ''

  # string or a sequence
  return ob

def _replace(s, replace_map):
  for orig, repl in replace_map:
    s = s.replace(orig, repl)
  return s

REPLACE_JS_MAP = (
  ('\\', r'\\'), ('\t', r'\t'), ('\n', r'\n'), ('\r', r'\r'),
  ('"', r'\x22'), ('\'', r'\x27'), ('&', r'\x26'),
  ('<', r'\x3c'), ('>', r'\x3e'), ('=', r'\x3d'),
)

# Various unicode whitespace
if sys.version_info[0] >= 3:
  # Python >=3.0
  REPLACE_JS_UNICODE_MAP = (
    ('\u0085', r'\u0085'), ('\u2028', r'\u2028'), ('\u2029', r'\u2029')
  )
else:
  # Python <3.0
  REPLACE_JS_UNICODE_MAP = eval("((u'\u0085', r'\u0085'), (u'\u2028', r'\u2028'), (u'\u2029', r'\u2029'))")

# Why not cgi.escape? It doesn't do single quotes which are occasionally
# used to contain HTML attributes and event handler definitions (unfortunately)
REPLACE_HTML_MAP = (
  ('&', '&amp;'), ('<', '&lt;'), ('>', '&gt;'),
  ('"', '&quot;'), ('\'', '&#39;'),
)

def _js_escape(s):
  s = _replace(s, REPLACE_JS_MAP)
  ### perhaps attempt to coerce the string to unicode and then replace?
  if isinstance(s, unicode):
    s = _replace(s, REPLACE_JS_UNICODE_MAP)
  return s

def _html_escape(s):
  return _replace(s, REPLACE_HTML_MAP)

def _url_escape(s):
  ### quote_plus barfs on non-ASCII characters. According to
  ### http://www.w3.org/International/O-URL-code.html URIs should be
  ### UTF-8 encoded first.
  if isinstance(s, unicode):
    s = s.encode('utf8')
  return urllib_parse_quote_plus(s)

FORMATTERS = {
  FORMAT_RAW: None,
  FORMAT_HTML: _html_escape,
  FORMAT_XML: _html_escape,   ### use the same quoting as HTML for now
  FORMAT_JS: _js_escape,
  FORMAT_URL: _url_escape,
}

def _parse_format(format_string=FORMAT_RAW):
  format_funcs = []
  try:
    for fspec in format_string.split(','):
      format_func = FORMATTERS[fspec]
      if format_func is not None:
        format_funcs.append(format_func)
  except KeyError:
    raise UnknownFormatConstantError(format_string)
  return format_funcs

class _context:
  """A container for the execution context"""


class Reader:
  """Abstract class which allows EZT to detect Reader objects."""
  def filename(self):
    return '(%s does not provide filename() method)' % repr(self)

class _FileReader(Reader):
  """Reads templates from the filesystem."""
  def __init__(self, fname):
    self.text = open(fname, 'rb').read()
    if sys.version_info[0] >= 3:
      # Python >=3.0
      self.text = self.text.decode()
    self._dir = os.path.dirname(fname)
    self.fname = fname
  def read_other(self, relative):
    return _FileReader(os.path.join(self._dir, relative))
  def filename(self):
    return self.fname

class _TextReader(Reader):
  """'Reads' a template from provided text."""
  def __init__(self, text):
    self.text = text
  def read_other(self, relative):
    raise BaseUnavailableError()
  def filename(self):
    return '(text)'


class EZTException(Exception):
  """Parent class of all EZT exceptions."""
  def __init__(self, message=None, filename=None, line_number=None):
    self.message = message
    self.filename = filename
    self.line_number = line_number
  def __str__(self):
    ret = []
    if self.message is not None:
      ret.append(self.message)
    if self.filename is not None:
      ret.append('in file ' + str(self.filename))
    if self.line_number is not None:
      ret.append('at line ' + str(self.line_number))
    return ' '.join(ret)

class ArgCountSyntaxError(EZTException):
  """A bracket directive got the wrong number of arguments."""

class UnknownReference(EZTException):
  """The template references an object not contained in the data dictionary."""

class NeedSequenceError(EZTException):
  """The object dereferenced by the template is no sequence (tuple or list)."""

class UnclosedBlocksError(EZTException):
  """This error may be simply a missing [end]."""

class UnmatchedEndError(EZTException):
  """This error may be caused by a misspelled if directive."""

class BaseUnavailableError(EZTException):
  """Base location is unavailable, which disables includes."""

class BadFormatConstantError(EZTException):
  """Format specifiers must be string constants."""

class UnknownFormatConstantError(EZTException):
  """The format specifier is an unknown value."""


# --- standard test environment ---
def test_parse():
  assert _re_parse.split('[a]') == ['', '[a]', None, '']
  assert _re_parse.split('[a] [b]') == \
         ['', '[a]', None, ' ', '[b]', None, '']
  assert _re_parse.split('[a c] [b]') == \
         ['', '[a c]', None, ' ', '[b]', None, '']
  assert _re_parse.split('x [a] y [b] z') == \
         ['x ', '[a]', None, ' y ', '[b]', None, ' z']
  assert _re_parse.split('[a "b" c "d"]') == \
         ['', '[a "b" c "d"]', None, '']
  assert _re_parse.split(r'["a \"b[foo]" c.d f]') == \
         ['', '["a \\"b[foo]" c.d f]', None, '']

def _test(argv):
  import doctest, ezt
  verbose = "-v" in argv
  return doctest.testmod(ezt, verbose=verbose)

if __name__ == "__main__":
  # invoke unit test for this module:
  import sys
  sys.exit(_test(sys.argv)[0])



#!/usr/bin/env python
#
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#

import os, re, sys

msgstr_re = re.compile('msgstr\[\d+\] "')

def parse_translation(f):
    """Read a single translation entry from the file F and return a
    tuple with the comments, msgid, msgid_plural and msgstr.  The comments is
    returned as a list of lines which do not end in new-lines.  The msgid is
    string.  The msgid_plural is string or None.  The msgstr is a list of
    strings.  The msgid, msgid_plural and msgstr strings can contain embedded
    newlines"""
    line = f.readline()

    # Parse comments
    comments = []
    while True:
        if line.strip() == '' or line[:2] == '#~':
            return comments, None, None, None
        elif line[0] == '#':
            comments.append(line[:-1])
        else:
            break
        line = f.readline()

    # Parse msgid
    if line[:7] != 'msgid "' or line[-2] != '"':
        raise RuntimeError("parse error")
    msgid = line[6:-1]
    while True:
        line = f.readline()
        if line[0] != '"':
            break
        msgid = msgid[:-1] + line[1:-1]

    # Parse optional msgid_plural
    msgid_plural = None
    if line[:14] == 'msgid_plural "':
        if line[-2] != '"':
            raise RuntimeError("parse error")
        msgid_plural = line[13:-1]
        while True:
            line = f.readline()
            if line[0] != '"':
                break
            msgid_plural = msgid_plural[:-1] + line[1:-1]

    # Parse msgstr
    msgstr = []
    if not msgid_plural:
        if line[:8] != 'msgstr "' or line[-2] != '"':
            raise RuntimeError("parse error")
        msgstr.append(line[7:-1])
        while True:
            line = f.readline()
            if len(line) == 0 or line[0] != '"':
                break
            msgstr[0] += '\n' + line[:-1]
    else:
        if line[:7] != 'msgstr[' or line[-2] != '"':
            raise RuntimeError("parse error")
        i = 0
        while True:
            matched_msgstr = msgstr_re.match(line)
            if matched_msgstr:
                matched_msgstr_len = len(matched_msgstr.group(0))
                msgstr.append(line[matched_msgstr_len-1:-1])
            else:
                break
            while True:
                line = f.readline()
                if len(line) == 0 or line[0] != '"':
                    break
                msgstr[i] += '\n' + line[:-1]
            i += 1

    if line.strip() != '':
        raise RuntimeError("parse error")

    return comments, msgid, msgid_plural, msgstr

def split_comments(comments):
    """Split COMMENTS into flag comments and other comments.  Flag
    comments are those that begin with '#,', e.g. '#,fuzzy'."""
    flags = []
    other = []
    for c in comments:
        if len(c) > 1 and c[1] == ',':
            flags.append(c)
        else:
            other.append(c)
    return flags, other

def main(argv):
    if len(argv) != 2:
        argv0 = os.path.basename(argv[0])
        sys.exit('Usage: %s <lang.po>\n'
                 '\n'
                 'This script will replace the translations and flags in lang.po (LF line endings)\n'
                 'with the translations and flags in the source po file read from standard input.\n'
                 'Strings that are not found in the source file are left untouched.\n'
                 'A backup copy of lang.po is saved as lang.po.bak.\n'
                 '\n'
                 'Example:\n'
                 '    svn cat http://svn.apache.org/repos/asf/subversion/trunk/subversion/po/sv.po | \\\n'
                 '        %s sv.po' % (argv0, argv0))

    # Read the source po file into a hash
    source = {}
    while True:
        comments, msgid, msgid_plural, msgstr = parse_translation(sys.stdin)
        if not comments and msgid is None:
            break
        if msgid is not None:
            source[msgid] = msgstr, split_comments(comments)[0]

    # Make a backup of the output file, open the copy for reading
    # and the original for writing.
    os.rename(argv[1], argv[1] + '.bak')
    infile = open(argv[1] + '.bak')
    outfile = open(argv[1], 'w')

    # Loop thought the original and replace stuff as we go
    first = 1
    string_count = 0
    update_count = 0
    untranslated = 0
    fuzzy = 0
    while True:
        comments, msgid, msgid_plural, msgstr = parse_translation(infile)
        if not comments and msgid is None:
            break
        if not first:
            outfile.write('\n')
        first = 0
        if msgid is None:
            outfile.write('\n'.join(comments) + '\n')
        else:
            string_count += 1
            # Do not update the header, and only update if the source
            # has a non-empty translation.
            if msgid != '""' and source.get(msgid, ['""', []])[0] != '""':
                other = split_comments(comments)[1]
                new_msgstr, new_flags = source[msgid]
                new_comments = other + new_flags
                if new_msgstr != msgstr or new_comments != comments:
                    update_count += 1
                    msgstr = new_msgstr
                    comments = new_comments
            outfile.write('\n'.join(comments) + '\n')
            outfile.write('msgid ' + msgid + '\n')
            if not msgid_plural:
                outfile.write('msgstr ' + msgstr[0] + '\n')
            else:
                outfile.write('msgid_plural ' + msgid_plural + '\n')
                n = 0
                for i in msgstr:
                    outfile.write('msgstr[%s] %s\n' % (n, msgstr[n]))
                    n += 1
        if msgstr is not None:
            for m in msgstr:
                if m == '""':
                    untranslated += 1
        for c in comments:
            if c.startswith('#,') and 'fuzzy' in c.split(', '):
                fuzzy += 1

    # We're done.  Tell the user what we did.
    print(('%d strings updated. '
          '%d fuzzy strings. '
          '%d of %d strings are still untranslated (%.0f%%).' %
          (update_count, fuzzy, untranslated, string_count,
           100.0 * untranslated / string_count)))

if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/env python
#
#  svnmucc_tests.py: tests of svnmucc
#
#  Subversion is a tool for revision control.
#  See http://subversion.apache.org for more information.
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################

import svntest
import re

XFail = svntest.testcase.XFail_deco
Issues = svntest.testcase.Issues_deco
Issue = svntest.testcase.Issue_deco

######################################################################

@Issues(3895,3953)
def reject_bogus_mergeinfo(sbox):
  "reject bogus mergeinfo"

  sbox.build(create_wc=False)

  expected_error = ".*(E200020.*Invalid revision|E175002.*PROPPATCH)"

  # At present this tests the server, but if we ever make svnmucc
  # validate the mergeinfo up front then it will only test the client
  svntest.actions.run_and_verify_svnmucc([], expected_error,
                                         'propset', 'svn:mergeinfo', '/B:0',
                                         '-m', 'log msg',
                                         sbox.repo_url + '/A')

_svnmucc_re = re.compile(b'^(r[0-9]+) committed by jrandom at (.*)$')
_log_re = re.compile(r'^   ([ADRM] /[^\(]+($| \(from .*:[0-9]+\)$))')
_err_re = re.compile('^svnmucc: (.*)$')

def test_svnmucc(repo_url, expected_path_changes, *varargs):
  """Run svnmucc with the list of SVNMUCC_ARGS arguments.  Verify that
  its run results in a new commit with 'svn log -rHEAD' changed paths
  that match the list of EXPECTED_PATH_CHANGES."""

  # First, run svnmucc.
  exit_code, outlines, errlines = svntest.main.run_svnmucc('-U', repo_url,
                                                           *varargs)
  if errlines:
    raise svntest.main.SVNCommitFailure(str(errlines))
  if len(outlines) != 1 or not _svnmucc_re.match(outlines[0]):
    raise svntest.main.SVNLineUnequal(str(outlines))

  # Now, run 'svn log -vq -rHEAD'
  changed_paths = []
  exit_code, outlines, errlines = \
    svntest.main.run_svn(None, 'log', '-vqrHEAD', repo_url)
  if errlines:
    raise svntest.Failure("Unable to verify commit with 'svn log': %s"
                          % (str(errlines)))
  for line in outlines:
    match = _log_re.match(line)
    if match:
      changed_paths.append(match.group(1).rstrip('\n\r'))

  expected_path_changes.sort()
  changed_paths.sort()
  if changed_paths != expected_path_changes:
    raise svntest.Failure("Logged path changes differ from expectations\n"
                          "   expected: %s\n"
                          "     actual: %s" % (str(expected_path_changes),
                                               str(changed_paths)))

def xtest_svnmucc(repo_url, expected_errors, *varargs):
  """Run svnmucc with the list of SVNMUCC_ARGS arguments.  Verify that
  its run results match the list of EXPECTED_ERRORS."""

  # First, run svnmucc.
  exit_code, outlines, errlines = svntest.main.run_svnmucc('-U', repo_url,
                                                           *varargs)
  errors = []
  for line in errlines:
    match = _err_re.match(line)
    if match:
      errors.append(line.rstrip('\n\r'))
  if errors != expected_errors:
    raise svntest.main.SVNUnmatchedError(str(errors))


def basic_svnmucc(sbox):
  "basic svnmucc tests"

  sbox.build()
  empty_file = sbox.ospath('empty')
  file = sbox.ospath('file')
  svntest.main.file_append(empty_file, '')
  svntest.main.file_append(file, 'file')

  # revision 2
  test_svnmucc(sbox.repo_url,
               ['A /foo'
                ], # ---------
               '-m', 'log msg',
               'mkdir', 'foo')

  # revision 3
  test_svnmucc(sbox.repo_url,
               ['A /z.c',
                ], # ---------
               '-m', 'log msg',
               'put', empty_file, 'z.c')

  # revision 4
  test_svnmucc(sbox.repo_url,
               ['A /foo/z.c (from /z.c:3)',
                'A /foo/bar (from /foo:3)',
                ], # ---------
               '-m', 'log msg',
               'cp', '3', 'z.c', 'foo/z.c',
               'cp', '3', 'foo', 'foo/bar')

  # revision 5
  test_svnmucc(sbox.repo_url,
               ['A /zig (from /foo:4)',
                'D /zig/bar',
                'D /foo',
                'A /zig/zag (from /foo:4)',
                ], # ---------
               '-m', 'log msg',
               'cp', '4', 'foo', 'zig',
               'rm',             'zig/bar',
               'mv',      'foo', 'zig/zag')

  # revision 6
  test_svnmucc(sbox.repo_url,
               ['D /z.c',
                'A /zig/zag/bar/y.c (from /z.c:5)',
                'A /zig/zag/bar/x.c (from /z.c:3)',
                ], # ---------
               '-m', 'log msg',
               'mv',      'z.c', 'zig/zag/bar/y.c',
               'cp', '3', 'z.c', 'zig/zag/bar/x.c')

  # revision 7
  test_svnmucc(sbox.repo_url,
               ['D /zig/zag/bar/y.c',
                'A /zig/zag/bar/y y.c (from /zig/zag/bar/y.c:6)',
                'A /zig/zag/bar/y%20y.c (from /zig/zag/bar/y.c:6)',
                ], # ---------
               '-m', 'log msg',
               'mv',         'zig/zag/bar/y.c', 'zig/zag/bar/y%20y.c',
               'cp', 'HEAD', 'zig/zag/bar/y.c', 'zig/zag/bar/y%2520y.c')

  # revision 8
  test_svnmucc(sbox.repo_url,
               ['D /zig/zag/bar/y y.c',
                'A /zig/zag/bar/z z1.c (from /zig/zag/bar/y y.c:7)',
                'A /zig/zag/bar/z%20z.c (from /zig/zag/bar/y%20y.c:7)',
                'A /zig/zag/bar/z z2.c (from /zig/zag/bar/y y.c:7)',
                ], #---------
               '-m', 'log msg',
               'mv',         'zig/zag/bar/y%20y.c',   'zig/zag/bar/z z1.c',
               'cp', 'HEAD', 'zig/zag/bar/y%2520y.c', 'zig/zag/bar/z%2520z.c',
               'cp', 'HEAD', 'zig/zag/bar/y y.c',     'zig/zag/bar/z z2.c')


  # revision 9
  test_svnmucc(sbox.repo_url,
               ['D /zig/zag',
                'A /zig/foo (from /zig/zag:8)',
                'D /zig/foo/bar/z%20z.c',
                'D /zig/foo/bar/z z2.c',
                'R /zig/foo/bar/z z1.c (from /zig/zag/bar/x.c:6)',
                ], #---------
               '-m', 'log msg',
               'mv',      'zig/zag',         'zig/foo',
               'rm',                         'zig/foo/bar/z z1.c',
               'rm',                         'zig/foo/bar/z%20z2.c',
               'rm',                         'zig/foo/bar/z%2520z.c',
               'cp', '6', 'zig/zag/bar/x.c', 'zig/foo/bar/z%20z1.c')

  # revision 10
  test_svnmucc(sbox.repo_url,
               ['R /zig/foo/bar (from /zig/z.c:9)',
                ], #---------
               '-m', 'log msg',
               'rm',                 'zig/foo/bar',
               'cp', '9', 'zig/z.c', 'zig/foo/bar')

  # revision 11
  test_svnmucc(sbox.repo_url,
               ['R /zig/foo/bar (from /zig/foo/bar:9)',
                'D /zig/foo/bar/z z1.c',
                ], #---------
               '-m', 'log msg',
               'rm',                     'zig/foo/bar',
               'cp', '9', 'zig/foo/bar', 'zig/foo/bar',
               'rm',                     'zig/foo/bar/z%20z1.c')

  # revision 12
  test_svnmucc(sbox.repo_url,
               ['R /zig/foo (from /zig/foo/bar:11)',
                ], #---------
               '-m', 'log msg',
               'rm',                        'zig/foo',
               'cp', 'head', 'zig/foo/bar', 'zig/foo')

  # revision 13
  test_svnmucc(sbox.repo_url,
               ['D /zig',
                'A /foo (from /foo:4)',
                'A /foo/foo (from /foo:4)',
                'A /foo/foo/foo (from /foo:4)',
                'D /foo/foo/bar',
                'R /foo/foo/foo/bar (from /foo:4)',
                ], #---------
               '-m', 'log msg',
               'rm',             'zig',
               'cp', '4', 'foo', 'foo',
               'cp', '4', 'foo', 'foo/foo',
               'cp', '4', 'foo', 'foo/foo/foo',
               'rm',             'foo/foo/bar',
               'rm',             'foo/foo/foo/bar',
               'cp', '4', 'foo', 'foo/foo/foo/bar')

  # revision 14
  test_svnmucc(sbox.repo_url,
               ['A /boozle (from /foo:4)',
                'A /boozle/buz',
                'A /boozle/buz/nuz',
                ], #---------
               '-m', 'log msg',
               'cp',    '4', 'foo', 'boozle',
               'mkdir',             'boozle/buz',
               'mkdir',             'boozle/buz/nuz')

  # revision 15
  test_svnmucc(sbox.repo_url,
               ['A /boozle/buz/svnmucc-test.py',
                'A /boozle/guz (from /boozle/buz:14)',
                'A /boozle/guz/svnmucc-test.py',
                ], #---------
               '-m', 'log msg',
               'put',      empty_file,   'boozle/buz/svnmucc-test.py',
               'cp', '14', 'boozle/buz', 'boozle/guz',
               'put',      empty_file,   'boozle/guz/svnmucc-test.py')

  # revision 16
  test_svnmucc(sbox.repo_url,
               ['M /boozle/buz/svnmucc-test.py',
                'R /boozle/guz/svnmucc-test.py',
                ], #---------
               '-m', 'log msg',
               'put', empty_file, 'boozle/buz/svnmucc-test.py',
               'rm',              'boozle/guz/svnmucc-test.py',
               'put', empty_file, 'boozle/guz/svnmucc-test.py')

  # revision 17
  test_svnmucc(sbox.repo_url,
               ['R /foo/bar (from /foo/foo:16)'
                ], #---------
               '-m', 'log msg',
               'rm',                            'foo/bar',
               'cp', '16', 'foo/foo',           'foo/bar',
               'propset',  'testprop',  'true', 'foo/bar')

  # revision 18
  test_svnmucc(sbox.repo_url,
               ['M /foo/bar'
                ], #---------
               '-m', 'log msg',
               'propdel', 'testprop', 'foo/bar')

  # revision 19
  test_svnmucc(sbox.repo_url,
               ['M /foo/z.c',
                'M /foo/foo',
                ], #---------
               '-m', 'log msg',
               'propset', 'testprop', 'true', 'foo/z.c',
               'propset', 'testprop', 'true', 'foo/foo')

  # revision 20
  test_svnmucc(sbox.repo_url,
               ['M /foo/z.c',
                'M /foo/foo',
                ], #---------
               '-m', 'log msg',
               'propsetf', 'testprop', empty_file, 'foo/z.c',
               'propsetf', 'testprop', empty_file, 'foo/foo')

  # revision 21
  test_svnmucc(sbox.repo_url,
               ['M /foo/z.c',
                ], #---------
               '-m', 'log msg',
               'propset', 'testprop', 'false', 'foo/z.c',
               'put', file, 'foo/z.c')

  # Expected missing revision error
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200004: 'a' is not a revision"
                 ], #---------
                '-m', 'log msg',
                'cp', 'a', 'b')

  # Expected cannot be younger error
  xtest_svnmucc(sbox.repo_url,
                ['svnmucc: E160006: No such revision 42',
                 ], #---------
                '-m', 'log msg',
                'cp', '42', 'a', 'b')

  # Expected already exists error
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160020: Path 'foo' already exists",
                 ], #---------
                '-m', 'log msg',
                'cp', '17', 'a', 'foo')

  # Expected copy_src already exists error
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160020: Path 'a/bar' already exists",
                 ], #---------
                '-m', 'log msg',
                'cp', '17', 'foo', 'a',
                'cp', '17', 'foo/foo', 'a/bar')

  # Expected not found error
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160013: Path 'a' not found in revision 17",
                 ], #---------
                '-m', 'log msg',
                'cp', '17', 'a', 'b')


def propset_root_internal(sbox, target):
  ## propset on ^/
  svntest.actions.run_and_verify_svnmucc(None, [],
                                         '-m', 'log msg',
                                         'propset', 'foo', 'bar',
                                         target)
  svntest.actions.run_and_verify_svn('bar', [],
                                     'propget', '--no-newline', 'foo',
                                     target)

  ## propdel on ^/
  svntest.actions.run_and_verify_svnmucc(None, [],
                                         '-m', 'log msg',
                                         'propdel', 'foo',
                                         target)
  svntest.actions.run_and_verify_svn([],
                                     '.*W200017: Property.*not found',
                                     'propget', '--no-newline', 'foo',
                                     target)

@Issues(3663)
def propset_root(sbox):
  "propset/propdel on repos root"

  sbox.build(create_wc=False)
  propset_root_internal(sbox, sbox.repo_url)
  propset_root_internal(sbox, sbox.repo_url + '/iota')


def too_many_log_messages(sbox):
  "test log message mutual exclusivity checks"

  sbox.build() # would use read-only=True, but need a place to stuff msg_file
  msg_file = sbox.ospath('svnmucc_msg')
  svntest.main.file_append(msg_file, 'some log message')
  err_msg = ["svnmucc: E205000: --message (-m), --file (-F), and "
             "--with-revprop=svn:log are mutually exclusive"]

  xtest_svnmucc(sbox.repo_url, err_msg,
                '--non-interactive',
                '-m', 'log msg',
                '-F', msg_file,
                'mkdir', 'A/subdir')
  xtest_svnmucc(sbox.repo_url, err_msg,
                '--non-interactive',
                '-m', 'log msg',
                '--with-revprop', 'svn:log=proppy log message',
                'mkdir', 'A/subdir')
  xtest_svnmucc(sbox.repo_url, err_msg,
                '--non-interactive',
                '-F', msg_file,
                '--with-revprop', 'svn:log=proppy log message',
                'mkdir', 'A/subdir')
  xtest_svnmucc(sbox.repo_url, err_msg,
                '--non-interactive',
                '-m', 'log msg',
                '-F', msg_file,
                '--with-revprop', 'svn:log=proppy log message',
                'mkdir', 'A/subdir')

@Issues(3418)
def no_log_msg_non_interactive(sbox):
  "test non-interactive without a log message"

  sbox.build(create_wc=False)
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E205001: Cannot invoke editor to get log message "
                 "when non-interactive"
                 ], #---------
                '--non-interactive',
                'mkdir', 'A/subdir')


def nested_replaces(sbox):
  "nested replaces"

  sbox.build(create_wc=False)
  repo_url = sbox.repo_url
  svntest.actions.run_and_verify_svnmucc(None, [],
                           '-U', repo_url, '-m', 'r2: create tree',
                           'rm', 'A',
                           'rm', 'iota',
                           'mkdir', 'A', 'mkdir', 'A/B', 'mkdir', 'A/B/C',
                           'mkdir', 'M', 'mkdir', 'M/N', 'mkdir', 'M/N/O',
                           'mkdir', 'X', 'mkdir', 'X/Y', 'mkdir', 'X/Y/Z')
  svntest.actions.run_and_verify_svnmucc(None, [],
                           '-U', repo_url, '-m', 'r3: nested replaces',
                           *("""
rm A rm M rm X
cp HEAD X/Y/Z A cp HEAD A/B/C M cp HEAD M/N/O X
cp HEAD A/B A/B cp HEAD M/N M/N cp HEAD X/Y X/Y
rm A/B/C rm M/N/O rm X/Y/Z
cp HEAD X A/B/C cp HEAD A M/N/O cp HEAD M X/Y/Z
rm A/B/C/Y
                           """.split()))

  # ### TODO: need a smarter run_and_verify_log() that verifies copyfrom
  excaped = svntest.main.ensure_list(map(re.escape, [
    '   R /A (from /X/Y/Z:2)',
    '   A /A/B (from /A/B:2)',
    '   R /A/B/C (from /X:2)',
    '   R /M (from /A/B/C:2)',
    '   A /M/N (from /M/N:2)',
    '   R /M/N/O (from /A:2)',
    '   R /X (from /M/N/O:2)',
    '   A /X/Y (from /X/Y:2)',
    '   R /X/Y/Z (from /M:2)',
    '   D /A/B/C/Y',
  ]))
  expected_output = svntest.verify.UnorderedRegexListOutput(excaped
                    + ['^--*', '^r3.*', '^--*', '^Changed paths:',])
  svntest.actions.run_and_verify_svn(expected_output, [],
                                     'log', '-qvr3', repo_url)


def prohibited_deletes_and_moves(sbox):
  "test prohibited delete and move operations"

  # These action sequences were allowed in 1.8.13, but are prohibited in 1.9.x
  # and later.  Most of them probably indicate an inadvertent user mistake.
  # See dev@, 2015-05-11, "Re: Issue 4579 / svnmucc fails to process certain
  # deletes", <http://svn.haxx.se/dev/archive-2015-05/0038.shtml>

  sbox.build(read_only = True)
  svntest.main.file_write(sbox.ospath('file'), "New contents")

  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't delete node at 'iota'",
                 ], #---------
                '-m', 'r2: modify and delete /iota',
                'put', sbox.ospath('file'), 'iota',
                'rm', 'iota')

  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't delete node at 'iota'",
                 ], #---------
                '-m', 'r2: propset and delete /iota',
                'propset', 'prop', 'val', 'iota',
                'rm', 'iota')

  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160013: Can't delete node at 'iota' as it does "
                 "not exist",
                 ], #---------
                '-m', 'r2: delete and delete /iota',
                'rm', 'iota',
                'rm', 'iota')

  # Subversion 1.8.13 used to move /iota without applying the text change.
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't delete node at 'iota'",
                 ], #---------
                '-m', 'r2: modify and move /iota',
                'put', sbox.ospath('file'), 'iota',
                'mv', 'iota', 'iota2')

  # Subversion 1.8.13 used to move /A without applying the inner remove.
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't delete node at 'A'",
                 ], #---------
                '-m', 'r2: delete /A/B and move /A',
                'rm', 'A/B',
                'mv', 'A', 'A1')

def svnmucc_type_errors(sbox):
  "test type errors"

  sbox.build(read_only=True)

  sbox.simple_append('file', 'New contents')

  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160016: Can't operate on 'B' "
                "because 'A' is not a directory"],
                '-m', '',
                'put', sbox.ospath('file'), 'A',
                'mkdir', 'A/B',
                'propset', 'iota', 'iota', 'iota')

  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't delete node at 'A'"],
                '-m', '',
                'mkdir', 'A/Z',
                'put', sbox.ospath('file'), 'A')

  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160020: Path 'Z' already exists, or was created "
                 "by an earlier operation"],
                '-m', '',
                'mkdir', 'A/Z',
                'put', sbox.ospath('file'), 'A/Z')

def svnmucc_propset_and_put(sbox):
  "propset and put"

  sbox.build()

  sbox.simple_append('file', 'New contents')

  # First in the sane order: put, then propset
  xtest_svnmucc(sbox.repo_url,
                [],
                '-m', '',
                'put', sbox.ospath('file'), 't1',
                'propset', 't1', 't1', 't1')

  # And now in an impossible order: propset, then put
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't set properties at not existing 't2'"],
                '-m', '',
                'propset', 't2', 't2', 't2',
                'put', sbox.ospath('file'), 't2')

  # And if the target already exists (dir)
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't delete node at 'A'"],
                '-m', '',
                'propset', 'A', 'A', 'A',
                'put', sbox.ospath('file'), 'A')

  # And if the target already exists (file) # fixed in r1702467
  xtest_svnmucc(sbox.repo_url,
                [],
                '-m', '',
                'propset', 'iota', 'iota', 'iota',
                'put', sbox.ospath('file'), 'iota')

  # Put same file twice (non existing)
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E160020: Path 't3' already exists, or was created "
                 "by an earlier operation"],
                '-m', '',
                'put', sbox.ospath('file'), 't3',
                'put', sbox.ospath('file'), 't3')

  # Put same file twice (existing)
  xtest_svnmucc(sbox.repo_url,
                ["svnmucc: E200009: Can't update file at 't1'"],
                '-m', '',
                'put', sbox.ospath('file'), 't1',
                'put', sbox.ospath('file'), 't1')


######################################################################

test_list = [ None,
              reject_bogus_mergeinfo,
              basic_svnmucc,
              propset_root,
              too_many_log_messages,
              no_log_msg_non_interactive,
              nested_replaces,
              prohibited_deletes_and_moves,
              svnmucc_type_errors,
              svnmucc_propset_and_put,
            ]

if __name__ == '__main__':
  svntest.main.run_tests(test_list)



#
# get-py-info.py: get various Python info (for building)
#
######################################################################
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################
#
# This should be loaded/run by the appropriate Python, rather than executed
# directly as a program. In other words, you should:
#
#    $ python2 get-py-info.py --includes
#

import sys
import os

def usage():
  print('USAGE: python %s WHAT' % sys.argv[0])
  print('  Returns information about how to build Python extensions.')
  print('  WHAT may be one of:')
  print("    --includes : return -I include flags")
  print("    --compile  : return a compile command")
  print("    --link     : return a link command")
  print("    --libs     : return just the library options for linking")
  print("    --site     : return the path to site-packages")
  sys.exit(1)

if len(sys.argv) != 2:
  usage()

try:
  if sys.version_info[0] == 2:
    from distutils import sysconfig
    get_include = lambda: sysconfig.get_python_inc()
    get_platinclude = lambda: sysconfig.get_python_inc(plat_specific=1)
    get_purelib = lambda: sysconfig.get_python_lib()
  else:
    import sysconfig
    get_include = lambda: sysconfig.get_path('include')
    get_platinclude = lambda: sysconfig.get_path('platinclude')
    get_purelib = lambda: sysconfig.get_path('purelib')
except ImportError:
  # No information available
  print("none")
  sys.exit(1)

if sys.argv[1] == '--includes':
  inc = get_include()
  plat = get_platinclude()
  if inc == plat:
    print("-I" + inc)
  else:
    print("-I%s -I%s" % (inc, plat))
  sys.exit(0)

if sys.argv[1] == '--compile':
  cc, ccshared = sysconfig.get_config_vars('CC', 'CCSHARED')
  print("%s %s" % (cc, ccshared))
  sys.exit(0)

def add_option(options, name, value=None):
  """Add option to list of options"""
  options.append(name)
  if value is not None:
    options.append(value)

def add_option_if_missing(options, name, value=None):
  """Add option to list of options, if it is not already present"""
  if options.count(name) == 0 and options.count("-Wl,%s" % name) == 0:
    add_option(options, name, value)

def link_options():
  """Get list of Python linker options"""

  # Initialize config variables
  assert os.name == "posix"
  options = sysconfig.get_config_var('LDSHARED').split()

  if sys.platform == 'darwin':

    # Load bundles from python
    python_exe = os.path.join(sysconfig.get_config_var("BINDIR"),
      sysconfig.get_config_var('PYTHON'))
    add_option_if_missing(options, "-bundle_loader", python_exe)

  elif sys.platform == 'cygwin' or sys.platform.startswith('openbsd'):

    # Add flags to build against the Python library (also necessary
    # for Darwin, but handled elsewhere).

    # Find the path to the library, and add a flag to include it as a
    # library search path.
    shared_libdir = sysconfig.get_config_var('LIBDIR')
    static_libdir = sysconfig.get_config_var('LIBPL')
    ldlibrary = sysconfig.get_config_var('LDLIBRARY')
    if os.path.exists(os.path.join(shared_libdir, ldlibrary)):
      if shared_libdir != '/usr/lib':
        add_option_if_missing(options, '-L%s' % shared_libdir)
    elif os.path.exists(os.path.join(static_libdir, ldlibrary)):
      add_option_if_missing(options, "-L%s" % static_libdir)

    # Add a flag to build against the library itself.
    python_version = sysconfig.get_config_var('VERSION')
    add_option_if_missing(options, "-lpython%s" % python_version)

  return options

def lib_options():
  """Get list of Python library options"""
  link_command = link_options()
  options = []

  # Extract library-related options from link command
  for i in range(len(link_command)):
    option = link_command[i]
    if (not option.startswith("-L:") and option.startswith("-L") or
        option.startswith("-Wl,") or option.startswith("-l") or
        option.startswith("-F") or option == "-bundle" or
        option == "-flat_namespace"):
      options.append(option)
    elif (option == "-undefined" or option == "-bundle_loader" or
          option == "-framework"):
      options.append(option)
      options.append(link_command[i+1])

  return options

if sys.argv[1] == '--link':
  print(" ".join(link_options()))
  sys.exit(0)

if sys.argv[1] == '--libs':
  print(" ".join(lib_options()))
  sys.exit(0)

if sys.argv[1] == '--site':
  print(get_purelib())
  sys.exit(0)

usage()



#!/usr/bin/env python
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.

"""
Script to change old (svn.collab.net) revision numbers in subversion log
messages to new ASF subversion repository revision numbers.
"""

USAGE = """python log_revnum_change_asf.py [OPTION]... URL

Change the revision numbers relatively in the log messages of new ASF
subversion repository.
"""

from csvn.repos import RemoteRepository
from csvn.auth import User
import csvn.core
from optparse import OptionParser
import sys
import re

def repl_newrev(matchobj):
    """
    Revision to be substituted is provided here.
    """
    if matchobj.group(0):
        old_rev = int(matchobj.group(0)[1:])
        if old_rev <= 45000:
            return 'r'+str(old_rev + 840074)
        else:
            return 'r'+str(old_rev)

def main():
    """
    Script execution starts here.
    """

    parser = OptionParser(usage=USAGE)
    parser.add_option("-u", "", dest="username",
                      help="commit the changes as USERNAME")
    parser.add_option("-p", "", dest="password",
                      help="commit the changes with PASSWORD")
    parser.add_option("-r", "", dest="rev",
                      help="revision range")

    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.print_help()
        sys.exit(1)

    csvn.core.svn_cmdline_init("", csvn.core.stderr)
    repos_url = args[0]
    revs = options.rev
    if revs and ":" in revs:
        [start_rev, end_rev] = revs.split(":")
    elif revs:
        start_rev = revs
        end_rev = revs
    else:
        start_rev = 1
        end_rev = "HEAD"

    session = RemoteRepository(repos_url, user=User(options.username,
                                                    options.password))

    if end_rev == "HEAD":
        end_rev = session.latest_revnum()
    if start_rev == "HEAD":
        start_rev = session.latest_revnum()
    start_rev = int(start_rev)
    end_rev = int(end_rev)

    for entry in session.log(start_rev, end_rev):
        new_log = re.sub(r'(r\d+)', repl_newrev, entry.message)
        session.revprop_set(propname='svn:log',
                            propval=new_log,
                            revnum=entry.revision,
                            force=True)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
#  basic_tests.py:  testing working-copy interactions with ra_local
#
#  Subversion is a tool for revision control.
#  See http://subversion.apache.org for more information.
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################

# General modules
import shutil, stat, re, os, logging

logger = logging.getLogger()

# Our testing module
import svntest
from svntest import wc
from svntest import main
from svntest import actions

# (abbreviation)
Skip = svntest.testcase.Skip_deco
SkipUnless = svntest.testcase.SkipUnless_deco
XFail = svntest.testcase.XFail_deco
Issues = svntest.testcase.Issues_deco
Issue = svntest.testcase.Issue_deco
Wimp = svntest.testcase.Wimp_deco
Item = wc.StateItem

######################################################################
# Helper functions

# Most of our tests use absolute paths as parameters on the command line. But
# for these tests, it's important that we can use bare file names in the
# commands, because the parser may have (and as of this writing does have)
# edge-case bugs that we can only expose in this way. Therefore, these helpers
# ensure that we run 'svn' with the CWD at the root of the working copy.
def run_svn(sbox, expected_status, expected_stderr, *varargs):
  if expected_stderr is None:
    expected_stderr = []

  cwd = os.getcwd()
  try:
    os.chdir(sbox.wc_dir)
    actions.run_and_verify_svn(None, expected_stderr, *varargs)
  finally:
    os.chdir(cwd)

  if expected_status is not None:
    actions.run_and_verify_status(sbox.wc_dir, expected_status)

def get_trojan_virginal_state(sbox):
  return actions.get_virginal_state(sbox.wc_dir, '1', tree='trojan')

def build_trojan_sandbox(sbox, expected_stderr):
  sbox.build(tree='trojan')
  if expected_stderr is None:
    return get_trojan_virginal_state(sbox)
  return None

def build_empty_sandbox(sbox, expected_stderr):
  sbox.build(empty=True)
  if expected_stderr is None:
    return svntest.wc.State(sbox.wc_dir, {
      '': svntest.wc.StateItem(status='  ', wc_rev='0')
    })
  return None

def build_sandbox(sbox, empty_sandbox, expected_stderr):
  if not empty_sandbox:
    return build_trojan_sandbox(sbox, expected_stderr)
  else:
    return build_empty_sandbox(sbox, expected_stderr)

def do_add_file(sbox, dst, dst_cmdline,
                expected_stderr=None, empty_sandbox=False):
  expected_status = build_sandbox(sbox, empty_sandbox, expected_stderr)
  if expected_status is not None:
    expected_status.add({dst: Item(status='A ', wc_rev='-')})

  main.file_write(sbox.ospath(dst), "This is file '"  + dst + "'.")
  run_svn(sbox, expected_status, expected_stderr,
          'add', dst_cmdline)

def do_add_file_e(sbox, dst, dst_cmdline, expected_stderr=None):
  "like do_add_file() but with an empty sandbox"
  return do_add_file(sbox, dst, dst_cmdline, expected_stderr, True)

def do_make_dir(sbox, dst, dst_cmdline,
                expected_stderr=None, empty_sandbox=False):
  expected_status = build_sandbox(sbox, empty_sandbox, expected_stderr)
  if expected_status is not None:
    expected_status.add({dst: Item(status='A ', wc_rev='-')})

  run_svn(sbox, expected_status, expected_stderr,
          'mkdir', dst_cmdline)

def do_make_dir_e(sbox, dst, dst_cmdline, expected_stderr=None):
  "like do_make_dir() but with an empty sandbox"
  return do_make_dir(sbox, dst, dst_cmdline, expected_stderr, True)

def do_remove(sbox, dst, dst_cmdline, expected_stderr=None):
  expected_status = build_trojan_sandbox(sbox, expected_stderr)
  if expected_status is not None and dst in expected_status.desc:
    expected_status.tweak(dst, status='D ')

  run_svn(sbox, expected_status, expected_stderr,
          'remove', dst_cmdline)

def do_rename(sbox, src, src_cmdline, dst, dst_cmdline,
              expected_stderr=None):
  expected_status = build_trojan_sandbox(sbox, expected_stderr)
  if expected_status is not None:
    expected_status.tweak(src, status='D ', moved_to=dst)
    expected_status.add({dst: Item(status='A ', copied='+',
                                   moved_from=src, wc_rev='-')})

  run_svn(sbox, expected_status, expected_stderr,
          'rename', src_cmdline, dst_cmdline)


######################################################################
# Tests
#
#   Each test must return on success or raise on failure.

#=====================================================================
# Tests for 'svn add' in the current directory

def add_file_here_1_escape_peg(sbox):
  "add file 'tau' with pegrev escape"
  do_add_file_e(sbox, 'tau', 'tau@')

def add_file_here_2_escape_peg(sbox):
  "add file '@tau' with pegrev escape"
  do_add_file_e(sbox, '@tau', '@tau@')

def add_file_here_3_escape_peg(sbox):
  "add file '_@tau' with pegrev escape"
  do_add_file_e(sbox, '_@tau', '_@tau@')

def add_file_here_4_escape_peg(sbox):
  "add file '.@tau' with pegrev escape"
  do_add_file_e(sbox, '.@tau', '.@tau@')

def add_file_here_5_escape_peg(sbox):
  "add file 'tau@' with pegrev escape"
  do_add_file_e(sbox, 'tau@', 'tau@@')

def add_file_here_6_escape_peg(sbox):
  "add file '@tau@' with pegrev escape"
  do_add_file_e(sbox, '@tau@', '@tau@@')

def add_file_here_7_escape_peg(sbox):
  "add file '@' with pegrev escape"
  do_add_file_e(sbox, '@', '@@')

#---------------------------------------------------------------------

def add_file_here_1_no_escape_peg(sbox):
  "add file 'tau' without pegrev escape"
  do_add_file_e(sbox, 'tau', 'tau')

def add_file_here_2_no_escape_peg(sbox):
  "add file '@tau' without pegrev escape"
  do_add_file_e(sbox, '@tau', '@tau', "svn: E125001: '@tau'")

def add_file_here_3_no_escape_peg(sbox):
  "add file '_@tau' without pegrev escape"
  do_add_file_e(sbox, '_@tau', '_@tau', "svn: E200009: '_@tau'")

@Wimp("The error message mentions '@tau' instead of '.@tau'")
def add_file_here_4_no_escape_peg(sbox):
  "add file '.@tau' without pegrev escape"
  do_add_file_e(sbox, '.@tau', '.@tau', "svn: E200009: '.@tau'")

def add_file_here_5_no_escape_peg(sbox):
  "add file 'tau@' without pegrev escape"
  do_add_file_e(sbox, 'tau@', 'tau@', 'svn: E200009: ')

def add_file_here_6_no_escape_peg(sbox):
  "add file '@tau@' without pegrev escape"
  do_add_file_e(sbox, '@tau@', '@tau@', 'svn: E200009: ')

def add_file_here_7_no_escape_peg(sbox):
  "add file '@' without pegrev escape"
  do_add_file_e(sbox, '@', '@', "svn: E125001: '@'")

#=====================================================================
# Tests for 'svn add' in a subdirectory

def add_file_subdir_1_escape_peg(sbox):
  "add file 'E/tau' with pegrev escape"
  do_add_file(sbox, 'E/tau', 'E/tau@')

def add_file_subdir_2_escape_peg(sbox):
  "add file 'E/@tau' with pegrev escape"
  do_add_file(sbox, 'E/@tau', 'E/@tau@')

def add_file_subdir_3_escape_peg(sbox):
  "add file 'E/_@tau' with pegrev escape"
  do_add_file(sbox, 'E/_@tau', 'E/_@tau@')

def add_file_subdir_4_escape_peg(sbox):
  "add file 'E/.@tau' with pegrev escape"
  do_add_file(sbox, 'E/.@tau', 'E/.@tau@')

def add_file_subdir_5_escape_peg(sbox):
  "add file 'E/tau@' with pegrev escape"
  do_add_file(sbox, 'E/tau@', 'E/tau@@')

def add_file_subdir_6_escape_peg(sbox):
  "add file 'E/@tau@' with pegrev escape"
  do_add_file(sbox, 'E/@tau@', 'E/@tau@@')

def add_file_subdir_7_escape_peg(sbox):
  "add file 'E/@' with pegrev escape"
  do_add_file(sbox, 'E/@', 'E/@@')

#---------------------------------------------------------------------

def add_file_subdir_1_no_escape_peg(sbox):
  "add file 'E/tau' without pegrev escape"
  do_add_file(sbox, 'E/tau', 'E/tau')

@Wimp("The error message mentions 'E@tau' instead of 'E/@tau'")
@Wimp("The error message should be E125001")
def add_file_subdir_2_no_escape_peg(sbox):
  "add file 'E/@tau' without pegrev escape"
  do_add_file(sbox, 'E/@tau', 'E/@tau', r"svn: E200009: 'E[\\/]@tau'")

def add_file_subdir_3_no_escape_peg(sbox):
  "add file 'E/_@tau' without pegrev escape"
  do_add_file(sbox, 'E/_@tau', 'E/_@tau', r"svn: E200009: 'E[\\/]_@tau'")

@Wimp("The error message mentions 'E@tau' instead of 'E/.@tau'")
def add_file_subdir_4_no_escape_peg(sbox):
  "add file 'E/.@tau' without pegrev escape"
  do_add_file(sbox, 'E/.@tau', 'E/.@tau', r"svn: E200009: 'E[\\/].@tau'")

def add_file_subdir_5_no_escape_peg(sbox):
  "add file 'E/tau@' without pegrev escape"
  do_add_file(sbox, 'E/tau@', 'E/tau@', 'svn: E200009: ')

def add_file_subdir_6_no_escape_peg(sbox):
  "add file 'E/@tau@' without pegrev escape"
  do_add_file(sbox, 'E/@tau@', 'E/@tau@', 'svn: E200009: ')

@Wimp("The error message is E200009 but should be E125001")
def add_file_subdir_7_no_escape_peg(sbox):
  "add file 'E/@' without pegrev escape"
  do_add_file(sbox, 'E/@', 'E/@', r"svn: E125001: 'E[\\/]@'")

#=====================================================================
# Tests for 'svn mkdir' in the current directory

def make_dir_here_1_escape_peg(sbox):
  "create directory 'T' with pegrev escape"
  do_make_dir_e(sbox, 'T', 'T@')

def make_dir_here_2_escape_peg(sbox):
  "create directory '@T' with pegrev escape"
  do_make_dir_e(sbox, '@T', '@T@')

def make_dir_here_3_escape_peg(sbox):
  "create directory '_@T' with pegrev escape"
  do_make_dir(sbox, '_@T', '_@T@')

def make_dir_here_4_escape_peg(sbox):
  "create directory '.@T' with pegrev escape"
  do_make_dir_e(sbox, '.@T', '.@T@')

def make_dir_here_5_escape_peg(sbox):
  "create directory 'T@' with pegrev escape"
  do_make_dir_e(sbox, 'T@', 'T@@')

def make_dir_here_6_escape_peg(sbox):
  "create directory '@T@' with pegrev escape"
  do_make_dir_e(sbox, '@T@', '@T@@')

def make_dir_here_7_escape_peg(sbox):
  "create directory '@' with pegrev escape"
  do_make_dir_e(sbox, '@', '@@')

#---------------------------------------------------------------------

def make_dir_here_1_no_escape_peg(sbox):
  "create directory 'T' without pegrev escape"
  do_make_dir_e(sbox, 'T', 'T')

def make_dir_here_2_no_escape_peg(sbox):
  "create directory '@T' without pegrev escape"
  do_make_dir_e(sbox, '@T', '@T', "svn: E125001: '@T'")

def make_dir_here_3_no_escape_peg(sbox):
  "create directory '_@T' without pegrev escape"
  do_make_dir_e(sbox, '_@T', '_@T', "svn: E200009: '_@T'")

@Wimp("The error message mentions '@T' instead of '.@T'")
def make_dir_here_4_no_escape_peg(sbox):
  "create directory '.@T' without pegrev escape"
  do_make_dir_e(sbox, '.@T', '.@T', "svn: E200009: '.@T'")

# Skip tests 5 and 6 that create a directory with a trailing @ in the name
# because is correctly interpreted as a peg revision escape. This is already
# tested by:
#   - make_dir_here_5_escape_peg
#   - make_dir_here_6_escape_peg

def make_dir_here_7_no_escape_peg(sbox):
  "create directory '@' without pegrev escape"
  do_make_dir_e(sbox, '@', '@', "svn: E125001: '@'")

#=====================================================================
# Tests for 'svn add' in a subdirectory

def make_dir_subdir_1_escape_peg(sbox):
  "create directory 'E/T' with pegrev escape"
  do_make_dir(sbox, 'E/T', 'E/T@')

def make_dir_subdir_2_escape_peg(sbox):
  "create directory 'E/@T' with pegrev escape"
  do_make_dir(sbox, 'E/@T', 'E/@T@')

def make_dir_subdir_3_escape_peg(sbox):
  "create directory 'E/_@T' with pegrev escape"
  do_make_dir(sbox, 'E/_@T', 'E/_@T@')

def make_dir_subdir_4_escape_peg(sbox):
  "create directory 'E/.@T' with pegrev escape"
  do_make_dir(sbox, 'E/.@T', 'E/.@T@')

def make_dir_subdir_5_escape_peg(sbox):
  "create directory 'E/T@' with pegrev escape"
  do_make_dir(sbox, 'E/T@', 'E/T@@')

def make_dir_subdir_6_escape_peg(sbox):
  "create directory 'E/@T@' with pegrev escape"
  do_make_dir(sbox, 'E/@T@', 'E/@T@@')

def make_dir_subdir_7_escape_peg(sbox):
  "create directory 'E/@' with pegrev escape"
  do_make_dir(sbox, 'E/@', 'E/@@')

#---------------------------------------------------------------------

def make_dir_subdir_1_no_escape_peg(sbox):
  "create directory 'E/T' without pegrev escape"
  do_make_dir(sbox, 'E/T', 'E/T')

@Wimp("The error message mentions 'E@T' instead of 'E/@T'")
@Wimp("The error message should be E125001")
def make_dir_subdir_2_no_escape_peg(sbox):
  "create directory 'E/@T' without pegrev escape"
  do_make_dir(sbox, 'E/@T', 'E/@T', r"svn: E200009: 'E[\\/]@T'")

def make_dir_subdir_3_no_escape_peg(sbox):
  "create directory 'E/_@T' without pegrev escape"
  do_make_dir(sbox, 'E/_@T', 'E/_@T', r"svn: E200009: 'E[\\/]_@T'")

@Wimp("The error message mentions 'E@T' instead of 'E/.@T'")
def make_dir_subdir_4_no_escape_peg(sbox):
  "create directory 'E/.@T' without pegrev escape"
  do_make_dir(sbox, 'E/.@T', 'E/.@T', r"svn: E200009: 'E[\\/].@T'")

# Skip tests 5 and 6 that create a directory with a trailing @ in the name
# because is correctly interpreted as a peg revision escape. This is already
# tested by:
#   - make_dir_subdir_5_escape_peg
#   - make_dir_subdir_6_escape_peg

@Wimp("Reports error that E exists but should be E125001 for E/@")
def make_dir_subdir_7_no_escape_peg(sbox):
  "create directory 'E/@' without pegrev escape"
  do_make_dir(sbox, 'E/@', 'E/@', r"svn: E125001: 'E[\\/]@'")

#=====================================================================
# Tests for 'svn remove' in the current directory

def remove_here_1_escape_peg(sbox):
  "remove 'iota' with pegrev escape"
  do_remove(sbox, 'iota', 'iota@')

def remove_here_2_escape_peg(sbox):
  "remove '@zeta' with pegrev escape"
  do_remove(sbox, '@zeta', '@zeta@')

def remove_here_3_escape_peg(sbox):
  "remove '_@theta' with pegrev escape"
  do_remove(sbox, '_@theta', '_@theta@')

def remove_here_4_escape_peg(sbox):
  "remove '.@kappa' with pegrev escape"
  do_remove(sbox, '.@kappa', '.@kappa@')

def remove_here_5_escape_peg(sbox):
  "remove 'lambda@' with pegrev escape"
  do_remove(sbox, 'lambda@', 'lambda@@')

def remove_here_6_escape_peg(sbox):
  "remove '@omicron@' with pegrev escape"
  do_remove(sbox, '@omicron@', '@omicron@@')

def remove_here_7_escape_peg(sbox):
  "remove '@' with pegrev escape"
  do_remove(sbox, '@', '@@')

#---------------------------------------------------------------------

def remove_here_1_no_escape_peg(sbox):
  "remove 'iota' without pegrev escape"
  do_remove(sbox, 'iota', 'iota')

def remove_here_2_no_escape_peg(sbox):
  "remove '@zeta' without pegrev escape"
  do_remove(sbox, '@zeta', '@zeta', "svn: E125001: '@zeta'")

def remove_here_3_no_escape_peg(sbox):
  "remove '_@theta' without pegrev escape"
  do_remove(sbox, '_@theta', '_@theta', "svn: E200009: '_@theta'")

@Wimp("The error message mentions '@kappa' instead of '.@kappa'")
def remove_here_4_no_escape_peg(sbox):
  "remove '.@kappa' without pegrev escape"
  do_remove(sbox, '.@kappa', '.@kappa', "svn: E200009: '.@kappa'")

def remove_here_5_no_escape_peg(sbox):
  "remove 'lambda@' without pegrev escape"
  do_remove(sbox, 'lambda@', 'lambda@', 'svn: E200005: ')

def remove_here_6_no_escape_peg(sbox):
  "remove '@omicron@' without pegrev escape"
  do_remove(sbox, '@omicron@', '@omicron@', 'svn: E200005: ')

def remove_here_7_no_escape_peg(sbox):
  "remove '@' without pegrev escape"
  do_remove(sbox, '@', '@', "svn: E125001: '@'")

#=====================================================================
# Tests for 'svn remove' in a subdirectory directory

def remove_subdir_1_escape_peg(sbox):
  "remove 'A/alpha' with pegrev escape"
  do_remove(sbox, 'A/alpha', 'A/alpha@')

def remove_subdir_2_escape_peg(sbox):
  "remove 'B/@beta' with pegrev escape"
  do_remove(sbox, 'B/@beta', 'B/@beta@')

def remove_subdir_3_escape_peg(sbox):
  "remove 'G/_@gamma' with pegrev escape"
  do_remove(sbox, 'G/_@gamma', 'G/_@gamma@')

def remove_subdir_4_escape_peg(sbox):
  "remove 'D/.@delta' with pegrev escape"
  do_remove(sbox, 'D/.@delta', 'D/.@delta@')

def remove_subdir_5_escape_peg(sbox):
  "remove 'B/pi@' with pegrev escape"
  do_remove(sbox, 'B/pi@', 'B/pi@@')

def remove_subdir_6_escape_peg(sbox):
  "remove 'A/@omega@' with pegrev escape"
  do_remove(sbox, 'A/@omega@', 'A/@omega@@')

def remove_subdir_7_escape_peg(sbox):
  "remove 'B/@' with pegrev escape"
  do_remove(sbox, 'B/@', 'B/@@')

def remove_subdir_7a_escape_peg(sbox):
  "remove missing 'E/@' with pegrev escape"
  do_remove(sbox, 'E/@', 'E/@@', r"svn: E200005: '.*[\\/]E[\\/]@'")

def remove_subdir_7b_escape_peg(sbox):
  "remove missing '@/@' with pegrev escape"
  do_remove(sbox, '@/@@', '@/@@', r"svn: E200005: '.*[\\/]@[\\/]@'")

#---------------------------------------------------------------------

def remove_subdir_1_no_escape_peg(sbox):
  "remove 'A/alpha' without pegrev escape"
  do_remove(sbox, 'A/alpha', 'A/alpha')

@Wimp("The error message mentions 'B@beta' instead of 'B/@beta'")
@Wimp("The error message should be E125001")
def remove_subdir_2_no_escape_peg(sbox):
  "remove 'B/@beta' without pegrev escape"
  do_remove(sbox, 'B/@beta', 'B/@beta', r"svn: E200009: 'B[\\/]@beta'")

def remove_subdir_3_no_escape_peg(sbox):
  "remove 'G/_@gamma' without pegrev escape"
  do_remove(sbox, 'G/_@gamma', 'G/_@gamma', r"svn: E200009: 'G[\\/]_@gamma'")

@Wimp("The error message mentions 'D@delta' instead of 'D/.@delta'")
def remove_subdir_4_no_escape_peg(sbox):
  "remove 'D/.@delta' without pegrev escape"
  do_remove(sbox, 'D/.@delta', 'D/.@delta', "svn: E200009: 'D/.@delta'")

# Skip tests 5 and 6 that remove a node with a trailing @ in the name
# because is correctly interpreted as a peg revision escape. This is already
# tested by:
#   - remove_subdir_5_escape_peg
#   - remove_subdir__escape_peg

@Wimp("Removes B instead of reporting E125001 for B/@")
def remove_subdir_7_no_escape_peg(sbox):
  "remove 'B/@' without pegrev escape"
  do_remove(sbox, 'B/@', 'B/@') #, r"svn: E125001: 'B[\\/]@'")

@Wimp("Removes E instead of reporting ENOENT or E125001 for E/@")
def remove_subdir_7a_no_escape_peg(sbox):
  "remove missing 'E/@' without pegrev escape"
  do_remove(sbox, 'E/@', 'E/@') #, r"svn: E125001: 'E[\\/]@'")

@Wimp("Removes @ instead of reporting ENOENT or E125001 for @/@")
def remove_subdir_7b_no_escape_peg(sbox):
  "remove missing '@/@' without pegrev escape"
  do_remove(sbox, '@/@', '@/@') #, r"svn: E125001: '@[\\/]@'")

#=====================================================================
# Test for 'svn move' to a subdirectory

@Wimp("Rename creates 'E/@tau@' instead of '@/@tau'")
@Issue(4530)
def rename_to_subdir_2_dst_escape_peg(sbox):
  "rename 'iota' to 'E/@tau with pegrev escape"
  # NOTE: This rename succeeds, but creates E/@tau@ instead of E/@tau, even
  #       though it should strip away the pegrev escape from the target.
  do_rename(sbox, 'iota', 'iota', 'E/@tau', 'E/@tau@')

#---------------------------------------------------------------------

@Wimp("Rename creates 'E@tau' instead of failing")
@Issue(4530)
def rename_to_subdir_2_no_dst_escape_peg(sbox):
  "rename 'iota' to 'E/@tau without pegrev escape"
  # NOTE: This rename succeeds, but creates E@tau in the current directory,
  #       when instead it should fail with 'svn: E125001: ...'.
  do_rename(sbox, 'iota', 'iota', 'E/@tau', 'E/@tau') ### 'svn: E200009: '


########################################################################
# Run the tests

# list all tests here, starting with None:
test_list = [ None,
              add_file_here_1_escape_peg,
              add_file_here_2_escape_peg,
              add_file_here_3_escape_peg,
              add_file_here_4_escape_peg,
              add_file_here_5_escape_peg,
              add_file_here_6_escape_peg,
              add_file_here_7_escape_peg,

              add_file_here_1_no_escape_peg,
              add_file_here_2_no_escape_peg,
              add_file_here_3_no_escape_peg,
              add_file_here_4_no_escape_peg,
              add_file_here_5_no_escape_peg,
              add_file_here_6_no_escape_peg,
              add_file_here_7_no_escape_peg,

              add_file_subdir_1_escape_peg,
              add_file_subdir_2_escape_peg,
              add_file_subdir_3_escape_peg,
              add_file_subdir_4_escape_peg,
              add_file_subdir_5_escape_peg,
              add_file_subdir_6_escape_peg,
              add_file_subdir_7_escape_peg,

              add_file_subdir_1_no_escape_peg,
              add_file_subdir_2_no_escape_peg,
              add_file_subdir_3_no_escape_peg,
              add_file_subdir_4_no_escape_peg,
              add_file_subdir_5_no_escape_peg,
              add_file_subdir_6_no_escape_peg,
              add_file_subdir_7_no_escape_peg,

              make_dir_here_1_escape_peg,
              make_dir_here_2_escape_peg,
              make_dir_here_3_escape_peg,
              make_dir_here_4_escape_peg,
              make_dir_here_5_escape_peg,
              make_dir_here_6_escape_peg,
              make_dir_here_7_escape_peg,

              make_dir_here_1_no_escape_peg,
              make_dir_here_2_no_escape_peg,
              make_dir_here_3_no_escape_peg,
              make_dir_here_4_no_escape_peg,
              # skipped: make_dir_here_5_no_escape_peg
              # skipped: make_dir_here_6_no_escape_peg
              make_dir_here_7_no_escape_peg,

              make_dir_subdir_1_escape_peg,
              make_dir_subdir_2_escape_peg,
              make_dir_subdir_3_escape_peg,
              make_dir_subdir_4_escape_peg,
              make_dir_subdir_5_escape_peg,
              make_dir_subdir_6_escape_peg,
              make_dir_subdir_7_escape_peg,

              make_dir_subdir_1_no_escape_peg,
              make_dir_subdir_2_no_escape_peg,
              make_dir_subdir_3_no_escape_peg,
              make_dir_subdir_4_no_escape_peg,
              # skipped: make_dir_subdir_5_no_escape_peg
              # skipped: make_dir_subdir_6_no_escape_peg
              make_dir_subdir_7_no_escape_peg,

              remove_here_1_escape_peg,
              remove_here_2_escape_peg,
              remove_here_3_escape_peg,
              remove_here_4_escape_peg,
              remove_here_5_escape_peg,
              remove_here_6_escape_peg,
              remove_here_7_escape_peg,

              remove_here_1_no_escape_peg,
              remove_here_2_no_escape_peg,
              remove_here_3_no_escape_peg,
              remove_here_4_no_escape_peg,
              remove_here_5_no_escape_peg,
              remove_here_6_no_escape_peg,
              remove_here_7_no_escape_peg,

              remove_subdir_1_escape_peg,
              remove_subdir_2_escape_peg,
              remove_subdir_3_escape_peg,
              remove_subdir_4_escape_peg,
              remove_subdir_5_escape_peg,
              remove_subdir_6_escape_peg,
              remove_subdir_7_escape_peg,
              remove_subdir_7a_escape_peg,
              remove_subdir_7b_escape_peg,

              remove_subdir_1_no_escape_peg,
              remove_subdir_2_no_escape_peg,
              remove_subdir_3_no_escape_peg,
              remove_subdir_4_no_escape_peg,
              # skipped: remove_subdir_5_no_escape_peg,
              # skipped: remove_subdir_6_no_escape_peg,
              remove_subdir_7_no_escape_peg,
              remove_subdir_7a_no_escape_peg,
              remove_subdir_7b_no_escape_peg,

              rename_to_subdir_2_dst_escape_peg,
              rename_to_subdir_2_no_dst_escape_peg,
             ]

if __name__ == '__main__':
  svntest.main.run_tests(test_list)
  # NOTREACHED


### End of file.
